#!/usr/bin/python3
"""
    GeckoLogStruct - A class to manage the LogStruct for 'InYT-V2 v62'
"""

from . import (
    GeckoByteStructAccessor,
    GeckoWordStructAccessor,
    GeckoTimeStructAccessor,
    GeckoBoolStructAccessor,
    GeckoEnumStructAccessor,
    GeckoTempStructAccessor,
)

# Constants for this class
ACCPQI = "".join(
    chr(c)
    for c in [
        82,
        101,
        109,
        111,
        116,
        101,
        70,
        105,
        108,
        116,
        68,
        117,
        114,
        80,
        101,
        114,
        68,
        97,
        121,
    ]
)
ACMCVD = 346
ACQFFT = "".join(
    chr(c)
    for c in [83, 108, 97, 118, 101, 82, 101, 103, 80, 114, 111, 98, 101, 69, 114, 114]
)
AFIKJP = "".join(chr(c) for c in [70, 117, 108, 108, 79, 110])
AHEOCT = 1
AIVEMV = "".join(chr(c) for c in [67, 70, 71, 51, 49])
AJVDQL = "".join(chr(c) for c in [82, 104, 72, 119, 72, 76])
AKQXPI = "".join(chr(c) for c in [85, 100, 80, 49])
AKSTSE = "".join(chr(c) for c in [83, 79, 117, 116, 54])
AMJMAO = "".join(
    chr(c)
    for c in [83, 105, 100, 101, 72, 101, 97, 116, 105, 110, 103, 68, 101, 103, 71]
)
AOAWBS = "".join(
    chr(c)
    for c in [
        67,
        111,
        111,
        108,
        90,
        111,
        110,
        101,
        68,
        101,
        116,
        101,
        99,
        116,
        101,
        100,
    ]
)
AONPYY = "".join(
    chr(c) for c in [70, 105, 108, 116, 101, 114, 65, 99, 99, 101, 115, 115]
)
ASSAKQ = "".join(chr(c) for c in [79, 70, 70])
ATDZXN = 326
AWBSIR = "".join(
    chr(c)
    for c in [
        72,
        80,
        67,
        82,
        101,
        115,
        72,
        101,
        97,
        116,
        101,
        114,
        82,
        101,
        113,
        117,
        101,
        115,
        116,
    ]
)
AZMKQT = "".join(chr(c) for c in [67, 70, 71, 51])
BBEKBD = 456
BDFSRO = 458
BDJQRJ = "".join(chr(c) for c in [51, 50, 75])
BEKBDF = "".join(chr(c) for c in [67, 70, 71, 57])
BFEGZU = "".join(chr(c) for c in [67, 80])
BHZVOA = "".join(chr(c) for c in [83, 79, 117, 116, 49, 48, 67, 117, 114])
BIAMJM = "".join(
    chr(c) for c in [83, 108, 97, 118, 101, 79, 118, 101, 114, 84, 101, 109, 112]
)
BJEUTO = 297
BJLOIN = "".join(chr(c) for c in [67, 70, 71, 49, 56])
BLKXSJ = "".join(chr(c) for c in [67, 108, 101, 97, 110])
BMJVHF = 256
BQFYLJ = 379
BQNRXC = "".join(chr(c) for c in [80, 49, 72])
BQSNQL = 283
BSIRYX = "".join(
    chr(c)
    for c in [72, 80, 67, 67, 104, 105, 108, 108, 82, 101, 113, 117, 101, 115, 116]
)
BSKSOK = "".join(chr(c) for c in [85, 100, 65, 117, 120])
BSSUHB = "".join(chr(c) for c in [85, 110, 107, 110, 111, 119, 110])
BVWVUB = "".join(chr(c) for c in [105, 110, 67, 108, 101, 97, 114])
BWJYKL = "".join(
    chr(c) for c in [83, 108, 97, 118, 101, 83, 119, 109, 65, 99, 116, 105, 118, 101]
)
BXIBHZ = 342
BXTIAC = "".join(
    chr(c)
    for c in [83, 108, 97, 118, 101, 75, 105, 110, 78, 111, 70, 108, 111, 69, 114, 114]
)
BXYBQS = "".join(
    chr(c)
    for c in [84, 104, 101, 114, 109, 105, 115, 116, 97, 110, 99, 101, 69, 114, 114]
)
BYGDSB = "".join(chr(c) for c in [105, 110, 89, 84])
CBFEGZ = "".join(chr(c) for c in [66, 76])
CCPQIP = 266
CGETIX = "".join(chr(c) for c in [83, 79, 117, 116, 49, 48])
CHWDAF = "".join(chr(c) for c in [80, 51, 76])
CMCVDS = "".join(chr(c) for c in [83, 68, 105, 114, 101, 99, 116, 67, 117, 114])
CPQIPO = "".join(
    chr(c) for c in [69, 99, 111, 110, 111, 109, 121, 65, 99, 99, 101, 115, 115]
)
CQBMJV = 317
CQFFTT = "".join(
    chr(c) for c in [83, 108, 97, 118, 101, 82, 101, 103, 83, 108, 111, 112, 101]
)
CRTFMN = 293
CTHBSK = 307
CTTGCR = 256
CVDSSR = "".join(chr(c) for c in [83, 68, 105, 114, 101, 99, 116, 50, 67, 117, 114])
CVYYPI = "".join(chr(c) for c in [68, 101, 97, 108, 101, 114, 79, 112, 116])
CWAONP = "".join(
    chr(c) for c in [68, 101, 97, 108, 101, 114, 76, 111, 99, 107, 75, 101, 121]
)
CXQIEF = 4
CYWONF = 372
CZOLSI = "".join(
    chr(c) for c in [77, 79, 68, 69, 95, 49, 95, 65, 78, 68, 95, 50, 95, 70, 65, 73, 76]
)
DAFIKJ = "".join(chr(c) for c in [66, 76, 79])
DDPMXF = 462
DFSROG = "".join(chr(c) for c in [67, 70, 71, 49, 49])
DGKEAK = "".join(chr(c) for c in [83, 79, 117, 116, 53, 67, 117, 114])
DJQRJJ = "".join(chr(c) for c in [52, 56, 75])
DKHTZB = "".join(chr(c) for c in [67, 70, 71, 54])
DMPSCT = "".join(chr(c) for c in [76, 73])
DNIBXT = 311
DNQGVU = "".join(chr(c) for c in [75, 54, 48, 48, 76, 69])
DPMXFU = "".join(chr(c) for c in [67, 70, 71, 49, 53])
DQLAII = "".join(
    chr(c) for c in [82, 104, 72, 114, 75, 105, 110, 78, 111, 70, 108, 111]
)
DRXAIV = 477
DSBDJQ = 290
DSSRUR = "".join(chr(c) for c in [67, 70, 71, 48])
DUBSSU = "".join(chr(c) for c in [80, 97, 99, 107, 84, 121, 112, 101])
EAKSTS = 333
ECVYYP = "".join(
    chr(c) for c in [73, 110, 115, 116, 97, 108, 108, 101, 114, 79, 112, 116]
)
EEZFET = 474
EFJTAC = "".join(chr(c) for c in [65, 67, 84, 73, 86, 69])
EFXQGL = "".join(chr(c) for c in [85, 100, 80, 52])
EGZUQE = 3
EJNIBX = "".join(
    chr(c)
    for c in [
        83,
        108,
        97,
        118,
        101,
        72,
        116,
        114,
        50,
        83,
        117,
        115,
        112,
        101,
        110,
        100,
        66,
        121,
        80,
        119,
        114,
        77,
        110,
        103,
    ]
)
EKBDFS = 457
EKCWAO = "".join(
    chr(c) for c in [68, 101, 97, 108, 101, 114, 76, 111, 99, 107, 83, 101, 101, 100]
)
EKVKZI = "".join(
    chr(c)
    for c in [83, 108, 97, 118, 101, 75, 105, 110, 80, 117, 109, 112, 79, 102, 102]
)
ELHBQN = "".join(chr(c) for c in [83, 79, 117, 116, 49])
ELWUEU = "".join(chr(c) for c in [67, 70, 71, 50, 49])
EMCGET = "".join(chr(c) for c in [83, 79, 117, 116, 56])
EMVCYW = "".join(chr(c) for c in [105, 110, 70, 108, 111, 82, 97, 116, 105, 111])
EOCTHB = 308
ETDRXA = 476
ETIXQV = "".join(chr(c) for c in [83, 79, 117, 116, 49, 50])
EUHNNX = "".join(chr(c) for c in [67, 70, 71, 50, 51])
EUTOPH = 299
EXLSXU = "".join(chr(c) for c in [87, 97, 116, 101, 114, 102, 97, 108, 108])
EZFETD = "".join(chr(c) for c in [67, 70, 71, 50, 55])
FCRTFM = "".join(chr(c) for c in [80, 97, 99, 107, 67, 111, 114, 101, 82, 101, 118])
FEFJTA = "".join(chr(c) for c in [78, 69, 87])
FEGZUQ = "".join(chr(c) for c in [79, 51])
FETDRX = "".join(chr(c) for c in [67, 70, 71, 50, 56])
FFTTID = 285
FIKJPU = "".join(chr(c) for c in [65, 85, 88])
FJBIAM = 281
FMNHTB = "".join(
    chr(c) for c in [80, 97, 99, 107, 67, 111, 110, 102, 105, 103, 76, 105, 98]
)
FSROGM = 459
FTHECV = 319
FTSIFJ = "".join(
    chr(c)
    for c in [69, 120, 116, 80, 114, 111, 98, 101, 68, 101, 116, 101, 99, 116, 101, 100]
)
FTTIDU = "".join(chr(c) for c in [80, 97, 99, 107, 66, 111, 111, 116, 82, 101, 118])
FUBJLO = "".join(chr(c) for c in [67, 70, 71, 49, 55])
FXQGLR = "".join(chr(c) for c in [85, 100, 80, 53])
FYLJUI = "".join(chr(c) for c in [67, 72, 73, 76, 76, 95, 79, 78])
FZCZOL = "".join(chr(c) for c in [70, 76, 79, 87, 95, 79, 75])
FZDGKE = "".join(chr(c) for c in [83, 79, 117, 116, 52, 67, 117, 114])
GDSBDJ = "".join(chr(c) for c in [80, 97, 99, 107, 77, 101, 109, 82, 97, 110, 103, 101])
GETIXQ = "".join(chr(c) for c in [83, 79, 117, 116, 49, 49])
GKEAKS = 331
GLRAHE = "".join(chr(c) for c in [85, 100, 66, 76])
GMDDPM = 461
GQPLSP = "".join(
    chr(c) for c in [68, 105, 115, 112, 108, 97, 121, 101, 100, 84, 101, 109, 112, 71]
)
GSELHB = "".join(chr(c) for c in [82, 69, 83, 69, 84])
GTYIYW = "".join(chr(c) for c in [82, 101, 115, 116, 114, 105, 99, 116, 101, 100])
GVUNXN = "".join(chr(c) for c in [75, 54, 48, 48, 72, 69])
GYOUSP = "".join(chr(c) for c in [78, 79])
GZUQEX = "".join(chr(c) for c in [76, 49, 50, 48])
HBQNRX = "".join(chr(c) for c in [78, 65])
HBSKSO = 363
HBVWVU = "".join(chr(c) for c in [68, 74, 83, 52])
HBXIBH = "".join(chr(c) for c in [83, 79, 117, 116, 56, 67, 117, 114])
HECVYY = "".join(chr(c) for c in [])
HEOCTH = "".join(chr(c) for c in [85, 100, 76, 49, 50, 48])
HFTHEC = "".join(chr(c) for c in [77, 101, 110, 117])
HIUSOO = 369
HNNXWE = "".join(chr(c) for c in [67, 70, 71, 50, 52])
HTBJEU = 296
HTZBBE = "".join(chr(c) for c in [67, 70, 71, 55])
HUGTYI = "".join(chr(c) for c in [80, 97, 99, 107, 76, 111, 103, 84, 114, 105, 103])
HUOJRJ = 305
HWDAFI = "".join(chr(c) for c in [80, 52, 72])
HXEKVK = "".join(
    chr(c) for c in [83, 108, 97, 118, 101, 72, 116, 114, 83, 116, 117, 99, 107]
)
HZVOAC = 344
IACQFF = "".join(chr(c) for c in [83, 108, 97, 118, 101, 72, 76, 69, 114, 114])
IAMJMA = 352
IBHZVO = 343
IBXTIA = 314
IBXYBQ = "".join(
    chr(c) for c in [84, 104, 101, 114, 109, 70, 117, 115, 101, 69, 114, 114]
)
ICXQIE = 0
IDNIBX = "".join(chr(c) for c in [105, 110, 84, 67, 105, 112, 68, 101, 108, 97, 121])
IDUBSS = 288
IEFXQG = "".join(chr(c) for c in [85, 100, 80, 51])
IFJBIA = "".join(
    chr(c) for c in [80, 114, 111, 103, 69, 99, 111, 110, 65, 99, 116, 105, 118, 101]
)
IGYOUS = 313
IHBXIB = 341
IIDNIB = "".join(chr(c) for c in [70, 76, 67, 69, 114, 114])
IJUGSE = "".join(chr(c) for c in [80, 97, 99, 107, 82, 101, 115, 101, 116])
IKFWRK = "".join(chr(c) for c in [80, 82, 69, 83, 83, 85, 82, 69, 95, 69, 82, 82])
ILXWAJ = "".join(
    chr(c)
    for c in [
        83,
        108,
        97,
        118,
        101,
        65,
        109,
        98,
        105,
        97,
        110,
        116,
        79,
        72,
        76,
        101,
        118,
        101,
        108,
        50,
    ]
)
INEJNI = "".join(
    chr(c)
    for c in [
        83,
        108,
        97,
        118,
        101,
        72,
        116,
        114,
        83,
        117,
        115,
        112,
        101,
        110,
        100,
        66,
        121,
        80,
        119,
        114,
        77,
        110,
        103,
    ]
)
INELWU = "".join(chr(c) for c in [67, 70, 71, 50, 48])
IPIVLA = "".join(chr(c) for c in [81, 117, 105, 101, 116, 83, 116, 97, 116, 101])
IPMDMP = 375
IPOUYN = "".join(
    chr(c) for c in [82, 101, 109, 111, 116, 101, 69, 99, 111, 110, 68, 117, 114]
)
IRYXBQ = "".join(chr(c) for c in [72, 69, 65, 84])
IUSOOQ = "".join(chr(c) for c in [80, 49])
IUXFEF = "".join(chr(c) for c in [73, 68, 76, 69])
IVDNQG = "".join(chr(c) for c in [75, 52])
IVEMVC = 479
IVLASS = "".join(chr(c) for c in [78, 79, 84, 95, 83, 69, 84])
IXQVXO = 332
IYWSKW = "".join(chr(c) for c in [75, 101, 121, 112, 97, 100, 84, 121, 112, 101])
JBIAMJ = "".join(chr(c) for c in [69, 99, 111, 110, 65, 99, 116, 105, 118, 101])
JEUTOP = "".join(chr(c) for c in [80, 97, 99, 107, 67, 111, 110, 102, 82, 101, 118])
JHIUSO = "".join(chr(c) for c in [85, 100, 65, 117, 120, 84, 105, 109, 101])
JIGYOU = "".join(chr(c) for c in [83, 119, 109, 82, 105, 115, 107])
JJJVYF = "".join(chr(c) for c in [85, 76])
JJVYFC = "".join(chr(c) for c in [67, 69])
JLOINE = 466
JMAOAW = "".join(
    chr(c) for c in [73, 110, 71, 114, 105, 100, 68, 101, 116, 101, 99, 116, 101, 100]
)
JMCBFE = 260
JNIBXY = "".join(
    chr(c)
    for c in [
        83,
        105,
        108,
        101,
        110,
        116,
        77,
        111,
        100,
        101,
        65,
        99,
        116,
        105,
        118,
        101,
    ]
)
JPUNRJ = 321
JQRJJJ = "".join(chr(c) for c in [54, 52, 75])
JRJHIU = "".join(
    chr(c)
    for c in [85, 100, 87, 97, 116, 101, 114, 70, 97, 108, 108, 84, 105, 109, 101]
)
JTACCP = "".join(
    chr(c) for c in [82, 101, 109, 111, 116, 101, 70, 105, 108, 116, 68, 117, 114]
)
JUGSEL = 376
JUIKFW = "".join(
    chr(c) for c in [65, 77, 66, 73, 69, 78, 84, 95, 84, 69, 77, 80, 95, 69, 82, 82]
)
JUTYEK = "".join(chr(c) for c in [70, 85, 76, 76])
JVDQLA = "".join(
    chr(c) for c in [82, 104, 82, 101, 103, 80, 114, 111, 98, 101, 69, 114, 114]
)
JVHFTH = 274
JWMNZM = "".join(
    chr(c)
    for c in [
        70,
        105,
        108,
        116,
        83,
        117,
        115,
        112,
        101,
        110,
        100,
        101,
        100,
        66,
        121,
        69,
        114,
        114,
    ]
)
JYKLGQ = 355
JYMOUN = 272
JZTATD = "".join(chr(c) for c in [83, 79, 117, 116, 53])
KBDFSR = "".join(chr(c) for c in [67, 70, 71, 49, 48])
KCWAON = 365
KEAKST = "".join(chr(c) for c in [83, 79, 117, 116, 72, 116, 114, 67, 117, 114])
KFWRKI = "".join(chr(c) for c in [79, 84, 72, 69, 82, 95, 69, 82, 82])
KHTZBB = 454
KINEJN = "".join(
    chr(c)
    for c in [
        72,
        116,
        114,
        50,
        83,
        117,
        115,
        112,
        101,
        110,
        100,
        66,
        121,
        80,
        119,
        114,
        77,
        110,
        103,
    ]
)
KJPUNR = "".join(chr(c) for c in [83, 79, 117, 116, 50])
KLGQPL = "".join(
    chr(c) for c in [82, 101, 97, 108, 83, 101, 116, 80, 111, 105, 110, 116, 71]
)
KMLOIJ = "".join(chr(c) for c in [75, 101, 121, 112, 97, 100, 82, 101, 118])
KPHUOJ = 304
KQTDKH = 452
KQXPIC = 259
KSOKPH = "".join(chr(c) for c in [85, 100, 80, 117, 109, 112, 84, 105, 109, 101])
KSTSEM = 325
KVKZIL = 351
KWIVDN = "".join(chr(c) for c in [75, 56, 53])
KXSJWM = "".join(chr(c) for c in [80, 117, 114, 103, 101])
KZILXW = "".join(
    chr(c)
    for c in [
        83,
        108,
        97,
        118,
        101,
        84,
        104,
        101,
        114,
        109,
        70,
        117,
        115,
        101,
        69,
        114,
        114,
    ]
)
LAIIDN = "".join(
    chr(c) for c in [82, 104, 78, 111, 70, 108, 111, 88, 84, 114, 105, 101, 115]
)
LASSAK = "".join(chr(c) for c in [83, 79, 65, 75])
LGQPLS = 275
LHBQNR = 320
LIUXFE = 263
LJUIKF = "".join(
    chr(c) for c in [67, 79, 73, 76, 95, 83, 69, 78, 83, 79, 82, 95, 69, 82, 82]
)
LKXSJW = 273
LNMHXE = "".join(chr(c) for c in [82, 101, 108, 97, 121, 83, 116, 117, 99, 107])
LOIJUG = "".join(chr(c) for c in [75, 101, 121, 112, 97, 100, 82, 101, 108])
LOINEL = "".join(chr(c) for c in [67, 70, 71, 49, 57])
LRAHEO = "".join(chr(c) for c in [79, 78])
LSXUJU = "".join(chr(c) for c in [76, 111, 99, 107, 77, 111, 100, 101])
LWUEUH = 469
LXWAJV = "".join(
    chr(c)
    for c in [83, 108, 97, 118, 101, 77, 105, 115, 115, 105, 110, 103, 69, 114, 114]
)
MAOAWB = 384
MCGETI = "".join(chr(c) for c in [83, 79, 117, 116, 57])
MCVDSS = 347
MDDPMX = "".join(chr(c) for c in [67, 70, 71, 49, 52])
MFZDGK = 329
MHXEKV = "".join(
    chr(c) for c in [83, 108, 97, 118, 101, 80, 50, 72, 83, 116, 117, 99, 107]
)
MJIGYO = "".join(chr(c) for c in [83, 119, 109, 65, 99, 116, 105, 118, 101])
MJMAOA = 377
MJVHFT = "".join(
    chr(c) for c in [83, 116, 105, 99, 107, 68, 101, 116, 101, 99, 116, 101, 100]
)
MKQTDK = "".join(chr(c) for c in [67, 70, 71, 52])
MLOIJU = 360
MNHTBJ = 295
MNZMJI = "".join(
    chr(c) for c in [83, 119, 109, 80, 117, 114, 103, 101, 83, 117, 115, 112]
)
MOUNBL = "".join(
    chr(c)
    for c in [
        82,
        101,
        109,
        111,
        116,
        101,
        79,
        110,
        122,
        101,
        110,
        65,
        99,
        116,
        105,
        111,
        110,
    ]
)
MVCYWO = 371
MXFUBJ = "".join(chr(c) for c in [67, 70, 71, 49, 54])
NBLKXS = 381
NEJNIB = 350
NELWUE = 468
NFZCZO = 374
NHTBJE = "".join(
    chr(c) for c in [80, 97, 99, 107, 83, 116, 97, 116, 117, 115, 76, 105, 98]
)
NIBXTI = "".join(
    chr(c)
    for c in [83, 108, 97, 118, 101, 70, 108, 111, 68, 101, 116, 101, 99, 116, 101, 100]
)
NIBXYB = 383
NKMLOI = 358
NMHXEK = "".join(
    chr(c) for c in [83, 108, 97, 118, 101, 80, 49, 72, 83, 116, 117, 99, 107]
)
NNXWEE = 472
NPYYLI = "".join(chr(c) for c in [73, 78, 84, 69, 82, 78, 65, 76])
NQGVUN = "".join(chr(c) for c in [75, 49, 48, 48])
NQJYMO = 271
NQLNMH = "".join(chr(c) for c in [80, 50, 72, 83, 116, 117, 99, 107])
NQTMFZ = "".join(chr(c) for c in [83, 79, 117, 116, 50, 67, 117, 114])
NRJZTA = "".join(chr(c) for c in [83, 79, 117, 116, 52])
NRSJMC = "".join(chr(c) for c in [80, 51])
NRXCHW = "".join(chr(c) for c in [80, 50, 72])
NXWEEZ = "".join(chr(c) for c in [67, 70, 71, 50, 53])
NZMJIG = 282
OACMCV = "".join(chr(c) for c in [83, 79, 117, 116, 49, 50, 67, 117, 114])
OAWBSI = 378
OCTHBS = "".join(chr(c) for c in [85, 100, 76, 105])
OGMDDP = "".join(chr(c) for c in [67, 70, 71, 49, 51])
OIHBXI = "".join(chr(c) for c in [83, 79, 117, 116, 55, 67, 117, 114])
OIJUGS = 361
OINELW = 467
OJRJHI = 306
OKPHUO = "".join(chr(c) for c in [85, 100, 81, 117, 105, 101, 116, 84, 105, 109, 101])
OLSIPM = "".join(chr(c) for c in [72, 69, 65, 84, 73, 78, 71, 95, 70, 65, 73, 76])
ONFZCZ = "".join(
    chr(c) for c in [105, 110, 70, 108, 111, 69, 114, 114, 111, 114, 84, 121, 112, 101]
)
ONPYYL = 262
OOQNRS = "".join(chr(c) for c in [76, 79, 87])
OPHUGT = "".join(
    chr(c)
    for c in [80, 97, 99, 107, 78, 117, 109, 98, 101, 114, 79, 102, 67, 111, 110, 102]
)
OUNBLK = 380
OUSPBW = "".join(chr(c) for c in [69, 88, 84, 82, 69, 77, 69])
OUYNQJ = "".join(
    chr(c)
    for c in [
        82,
        101,
        109,
        111,
        116,
        101,
        67,
        111,
        110,
        102,
        105,
        103,
        73,
        110,
        100,
        101,
        120,
    ]
)
PBWJYK = 353
PFTSIF = 279
PHUGTY = 301
PHUOJR = "".join(chr(c) for c in [85, 100, 76, 105, 103, 104, 116, 84, 105, 109, 101])
PIVLAS = 257
PLSPFT = "".join(chr(c) for c in [72, 101, 97, 116, 105, 110, 103])
PMDMPS = "".join(
    chr(c) for c in [105, 110, 70, 108, 111, 74, 117, 115, 116, 82, 101, 115, 101, 116]
)
PMXFUB = 463
POUYNQ = 268
PQIPOU = "".join(
    chr(c)
    for c in [82, 101, 109, 111, 116, 101, 69, 99, 111, 110, 65, 99, 116, 105, 111, 110]
)
PUNRJZ = "".join(chr(c) for c in [83, 79, 117, 116, 51])
PYYLIU = "".join(chr(c) for c in [82, 69, 77, 79, 84, 69])
QBMJVH = "".join(chr(c) for c in [72, 111, 117, 114, 115])
QEXLSX = "".join(chr(c) for c in [83, 76, 86, 95, 72, 69, 65, 84, 69, 82])
QFFTTI = "".join(chr(c) for c in [80, 97, 99, 107, 66, 111, 111, 116, 73, 68])
QFYLJU = "".join(chr(c) for c in [72, 69, 65, 84, 95, 79, 78])
QGVUNX = "".join(chr(c) for c in [75, 56, 48, 48])
QIEFXQ = 2
QIPOUY = 267
QJYMOU = "".join(
    chr(c)
    for c in [
        82,
        101,
        109,
        111,
        116,
        101,
        66,
        114,
        101,
        97,
        107,
        101,
        114,
        73,
        110,
        100,
        101,
        120,
    ]
)
QLAIID = 309
QLNMHX = "".join(chr(c) for c in [72, 101, 97, 116, 101, 114, 83, 116, 117, 99, 107])
QNRSJM = "".join(chr(c) for c in [80, 50])
QNRXCH = "".join(chr(c) for c in [80, 49, 76])
QPLSPF = 277
QSNQLN = "".join(chr(c) for c in [82, 101, 103, 79, 118, 101, 114, 72, 101, 97, 116])
QTDKHT = "".join(chr(c) for c in [67, 70, 71, 53])
QTMFZD = 328
QVXOIH = "".join(chr(c) for c in [83, 68, 105, 114, 101, 99, 116, 50])
QXPICX = "".join(chr(c) for c in [76, 79])
RAZMKQ = 450
RJHIUS = 362
RJJJVY = "".join(chr(c) for c in [80, 97, 99, 107, 82, 101, 103, 105, 111, 110])
RJZTAT = 323
RKINEJ = 284
ROGMDD = 460
RSJMCB = "".join(chr(c) for c in [80, 52])
RTFMNH = "".join(chr(c) for c in [80, 97, 99, 107, 67, 111, 114, 101, 82, 101, 108])
RURAZM = 449
RXAIVE = "".join(chr(c) for c in [67, 70, 71, 51, 48])
RXCHWD = "".join(chr(c) for c in [80, 50, 76])
RYXBQF = "".join(chr(c) for c in [67, 72, 73, 76, 76])
SAKQXP = "".join(chr(c) for c in [65, 76, 76])
SBDJQR = "".join(chr(c) for c in [49, 54, 75])
SCTTGC = 62
SEMCGE = "".join(chr(c) for c in [83, 79, 117, 116, 55])
SIFJBI = 280
SIPMDM = "".join(
    chr(c) for c in [70, 111, 114, 99, 101, 67, 104, 101, 99, 107, 70, 108, 111]
)
SIRYXB = "".join(chr(c) for c in [72, 80, 67, 65, 117, 116, 111, 77, 111, 100, 101])
SJMCBF = "".join(chr(c) for c in [80, 53])
SJWMNZ = "".join(
    chr(c)
    for c in [
        70,
        105,
        108,
        116,
        83,
        117,
        115,
        112,
        101,
        110,
        100,
        101,
        100,
        66,
        121,
        79,
        84,
    ]
)
SKSOKP = 370
SKWIVD = "".join(chr(c) for c in [75, 52, 48, 48])
SNQLNM = "".join(chr(c) for c in [80, 49, 72, 83, 116, 117, 99, 107])
SOKPHU = 303
SOOQNR = "".join(chr(c) for c in [72, 73, 71, 72])
SPBWJY = "".join(
    chr(c) for c in [83, 108, 97, 118, 101, 83, 119, 109, 80, 117, 114, 103, 101]
)
SPFTSI = "".join(
    chr(c) for c in [84, 101, 109, 112, 78, 111, 116, 86, 97, 108, 105, 100]
)
SROGMD = "".join(chr(c) for c in [67, 70, 71, 49, 50])
SRURAZ = "".join(chr(c) for c in [67, 70, 71, 49])
SSRURA = 448
SSUHBV = "".join(chr(c) for c in [105, 110, 88, 69])
STSEMC = "".join(chr(c) for c in [70, 97, 110])
SUHBVW = "".join(chr(c) for c in [77, 97, 115, 73, 66, 67])
SXUJUT = 310
TACCPQ = 264
TATDZX = "".join(chr(c) for c in [83, 79, 117, 116, 72, 116, 114])
TBJEUT = "".join(chr(c) for c in [80, 97, 99, 107, 67, 111, 110, 102, 73, 68])
TDKHTZ = 453
TDRXAI = "".join(chr(c) for c in [67, 70, 71, 50, 57])
TDZXNQ = "".join(chr(c) for c in [72, 84, 82])
TFMNHT = 294
THBSKS = "".join(chr(c) for c in [85, 100, 87, 97, 116, 101, 114, 102, 97, 108, 108])
THECVY = "".join(chr(c) for c in [78, 79, 82, 77, 65, 76])
TIACQF = "".join(
    chr(c) for c in [83, 108, 97, 118, 101, 78, 111, 70, 108, 111, 69, 114, 114]
)
TIDUBS = "".join(chr(c) for c in [80, 97, 99, 107, 66, 111, 111, 116, 82, 101, 108])
TIXQVX = "".join(chr(c) for c in [83, 68, 105, 114, 101, 99, 116])
TMFZDG = "".join(chr(c) for c in [83, 79, 117, 116, 51, 67, 117, 114])
TOPHUG = 300
TSIFJB = "".join(chr(c) for c in [67, 104, 101, 99, 107, 70, 108, 111])
TTGCRH = 479
TTIDUB = 287
TYEKCW = "".join(
    chr(c)
    for c in [68, 101, 97, 108, 101, 114, 76, 111, 99, 107, 83, 116, 97, 116, 117, 115]
)
TYIYWS = "".join(chr(c) for c in [70, 117, 108, 108])
TZBBEK = 455
UBJLOI = 465
UBSSUH = 289
UBYGDS = "".join(chr(c) for c in [105, 110, 84, 111, 117, 99, 104])
UEUHNN = 470
UGSELH = "".join(chr(c) for c in [45, 45, 45])
UGTYIY = 316
UHBVWV = "".join(chr(c) for c in [77, 73, 65])
UHNNXW = 471
UIKFWR = "".join(chr(c) for c in [69, 69, 49, 95, 69, 69, 50, 95, 69, 82, 82])
UJUTYE = "".join(chr(c) for c in [80, 65, 82, 84, 73, 65, 76])
UNBLKX = "".join(
    chr(c) for c in [82, 101, 109, 111, 116, 101, 79, 110, 122, 101, 110, 68, 117, 114]
)
UNRJZT = 322
UNXNKM = "".join(chr(c) for c in [73, 78, 86, 65, 76, 73, 68, 95, 84, 89, 80, 69])
UOJRJH = "".join(chr(c) for c in [85, 100, 76, 49, 50, 48, 84, 105, 109, 101])
UQEXLS = 5
URAZMK = "".join(chr(c) for c in [67, 70, 71, 50])
USOOQN = 261
UTOPHU = "".join(chr(c) for c in [80, 97, 99, 107, 67, 111, 110, 102, 82, 101, 108])
UXFEFJ = "".join(chr(c) for c in [83, 84, 79, 80])
UYNQJY = 270
VCYWON = "".join(
    chr(c)
    for c in [
        105,
        110,
        70,
        108,
        111,
        82,
        97,
        116,
        105,
        111,
        77,
        105,
        110,
        105,
        109,
        117,
        109,
    ]
)
VDNQGV = "".join(chr(c) for c in [75, 53])
VDQLAI = "".join(chr(c) for c in [82, 104, 82, 101, 103, 83, 108, 111, 112, 101])
VDSSRU = 348
VEMVCY = "".join(
    chr(c)
    for c in [
        105,
        110,
        70,
        108,
        111,
        80,
        114,
        101,
        115,
        115,
        117,
        114,
        101,
        83,
        119,
        68,
        101,
        116,
        101,
        99,
        116,
        101,
        100,
    ]
)
VHFTHE = 6
VKZILX = "".join(
    chr(c)
    for c in [83, 108, 97, 118, 101, 82, 101, 103, 79, 118, 101, 114, 72, 101, 97, 116]
)
VLASSA = "".join(chr(c) for c in [68, 82, 65, 73, 78])
VOACMC = 345
VUBYGD = "".join(chr(c) for c in [105, 110, 84, 101, 114, 102, 97, 99, 101])
VUNXNK = "".join(chr(c) for c in [75, 51, 48, 48])
VWVUBY = "".join(chr(c) for c in [105, 110, 88, 77])
VXOIHB = "".join(chr(c) for c in [83, 79, 117, 116, 54, 67, 117, 114])
VYFCRT = "".join(chr(c) for c in [80, 97, 99, 107, 67, 111, 114, 101, 73, 68])
VYYPIP = "".join(
    chr(c) for c in [65, 99, 99, 101, 115, 115, 111, 114, 121, 79, 112, 116]
)
WAJVDQ = "".join(
    chr(c) for c in [82, 104, 70, 108, 111, 68, 101, 116, 101, 99, 116, 101, 100]
)
WAONPY = 367
WBSIRY = "".join(
    chr(c) for c in [72, 80, 67, 72, 101, 97, 116, 82, 101, 113, 117, 101, 115, 116]
)
WDAFIK = "".join(chr(c) for c in [80, 52, 76])
WEEZFE = "".join(chr(c) for c in [67, 70, 71, 50, 54])
WIVDNQ = "".join(chr(c) for c in [75, 56])
WJYKLG = "".join(chr(c) for c in [83, 119, 109, 65, 100, 99])
WMNZMJ = "".join(chr(c) for c in [67, 80, 79, 84])
WONFZC = 373
WRKINE = "".join(
    chr(c)
    for c in [
        72,
        116,
        114,
        83,
        117,
        115,
        112,
        101,
        110,
        100,
        66,
        121,
        80,
        119,
        114,
        77,
        110,
        103,
    ]
)
WSKWIV = "".join(chr(c) for c in [75, 50, 48, 48])
WUEUHN = "".join(chr(c) for c in [67, 70, 71, 50, 50])
WVUBYG = "".join(chr(c) for c in [75, 54, 48, 48])
XAIVEM = 478
XBQFYL = "".join(chr(c) for c in [72, 80, 67, 83, 116, 97, 116, 101])
XCHWDA = "".join(chr(c) for c in [80, 51, 72])
XEKVKZ = "".join(
    chr(c)
    for c in [83, 108, 97, 118, 101, 82, 101, 108, 97, 121, 83, 116, 117, 99, 107]
)
XFEFJT = "".join(chr(c) for c in [83, 84, 65, 82, 84])
XFUBJL = 464
XIBHZV = "".join(chr(c) for c in [83, 79, 117, 116, 57, 67, 117, 114])
XLSXUJ = 7
XNKMLO = "".join(chr(c) for c in [75, 101, 121, 112, 97, 100, 73, 68])
XNQTMF = 327
XOIHBX = 340
XPICXQ = "".join(chr(c) for c in [72, 73])
XQGLRA = 258
XQIEFX = "".join(chr(c) for c in [85, 100, 80, 50])
XSJWMN = "".join(
    chr(c)
    for c in [70, 105, 108, 116, 83, 117, 115, 112, 101, 110, 100, 66, 121, 85, 68]
)
XTIACQ = 315
XUJUTY = "".join(chr(c) for c in [85, 78, 76, 79, 67, 75])
XWAJVD = 354
XWEEZF = 473
XYBQSN = "".join(
    chr(c) for c in [65, 109, 98, 105, 97, 110, 116, 79, 72, 76, 101, 118, 101, 108, 50]
)
YBQSNQ = "".join(chr(c) for c in [75, 105, 110, 80, 117, 109, 112, 79, 102, 102])
YEKCWA = 364
YFCRTF = 291
YKLGQP = "".join(chr(c) for c in [79, 118, 101, 114, 84, 101, 109, 112])
YLIUXF = "".join(
    chr(c)
    for c in [
        82,
        101,
        109,
        111,
        116,
        101,
        70,
        105,
        108,
        116,
        65,
        99,
        116,
        105,
        111,
        110,
    ]
)
YLJUIK = "".join(
    chr(c) for c in [73, 78, 86, 65, 76, 73, 68, 95, 82, 69, 81, 85, 69, 83, 84]
)
YMOUNB = "".join(chr(c) for c in [79, 110, 122, 101, 110, 65, 99, 99, 101, 115, 115])
YNQJYM = "".join(
    chr(c)
    for c in [82, 101, 109, 111, 116, 101, 78, 98, 79, 102, 80, 104, 97, 115, 101, 115]
)
YOUSPB = "".join(chr(c) for c in [77, 69, 68])
YPIPIV = "".join(chr(c) for c in [83, 116, 105, 99, 107, 66, 97, 110, 107])
YWONFZ = "".join(
    chr(c)
    for c in [
        105,
        110,
        70,
        108,
        111,
        82,
        97,
        116,
        105,
        111,
        77,
        97,
        120,
        105,
        109,
        117,
        109,
    ]
)
YWSKWI = 357
YYPIPI = "".join(
    chr(c) for c in [67, 111, 110, 102, 105, 103, 83, 101, 108, 101, 99, 116]
)
ZBBEKB = "".join(chr(c) for c in [67, 70, 71, 56])
ZCQBMJ = "".join(chr(c) for c in [82, 104, 87, 97, 116, 101, 114, 84, 101, 109, 112])
ZCZOLS = "".join(chr(c) for c in [77, 79, 68, 69, 95, 49, 95, 70, 65, 73, 76])
ZDGKEA = 330
ZFETDR = 475
ZILXWA = "".join(
    chr(c)
    for c in [
        83,
        108,
        97,
        118,
        101,
        84,
        104,
        101,
        114,
        109,
        105,
        115,
        116,
        97,
        110,
        99,
        101,
        69,
        114,
        114,
    ]
)
ZMJIGY = "".join(chr(c) for c in [83, 119, 109, 80, 117, 114, 103, 101])
ZMKQTD = 451
ZOLSIP = "".join(
    chr(c)
    for c in [
        77,
        79,
        68,
        69,
        95,
        49,
        95,
        65,
        78,
        68,
        95,
        72,
        69,
        65,
        84,
        73,
        78,
        71,
        95,
        70,
        65,
        73,
        76,
    ]
)
ZTATDZ = 324
ZUQEXL = "".join(chr(c) for c in [77, 83, 84, 82, 95, 72, 69, 65, 84, 69, 82])
ZVOACM = "".join(chr(c) for c in [83, 79, 117, 116, 49, 49, 67, 117, 114])
ZXNQTM = "".join(chr(c) for c in [83, 79, 117, 116, 49, 67, 117, 114])
AIIDNI = [HECVYY, LAIIDN, LAIIDN, LAIIDN]
DZXNQT = [
    HBQNRX,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    TDZXNQ,
]
FJTACC = [IUXFEF, UXFEFJ, XFEFJT, FEFJTA, EFJTAC]
FWRKIN = [
    IUXFEF,
    QFYLJU,
    FYLJUI,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    YLJUIK,
    LJUIKF,
    JUIKFW,
    UIKFWR,
    IKFWRK,
    KFWRKI,
]
IKJPUN = [
    HBQNRX,
    BQNRXC,
    QNRXCH,
    NRXCHW,
    RXCHWD,
    XCHWDA,
    CHWDAF,
    HWDAFI,
    WDAFIK,
    SJMCBF,
    DAFIKJ,
    BFEGZU,
    FEGZUQ,
    GZUQEX,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    AFIKJP,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    EXLSXU,
    FIKJPU,
]
JVYFCR = [JJJVYF, JJVYFC]
LSIPMD = [FZCZOL, ZCZOLS, CZOLSI, ZOLSIP, OLSIPM]
LSPFTS = [HECVYY, PLSPFT, PLSPFT, PLSPFT]
MCBFEG = [ASSAKQ, SOOQNR]
MDMPSC = []
MPSCTT = [
    IUSOOQ,
    QNRSJM,
    NRSJMC,
    RSJMCB,
    SJMCBF,
    CBFEGZ,
    BFEGZU,
    FEGZUQ,
    GZUQEX,
    ZUQEXL,
    QEXLSX,
    EXLSXU,
    LSXUJU,
    TYEKCW,
    EKCWAO,
    CWAONP,
    DMPSCT,
]
NXNKML = [
    WSKWIV,
    SKWIVD,
    KWIVDN,
    WIVDNQ,
    IVDNQG,
    VDNQGV,
    DNQGVU,
    NQGVUN,
    QGVUNX,
    HECVYY,
    HECVYY,
    HECVYY,
    GVUNXN,
    VUNXNK,
    UNXNKM,
]
OQNRSJ = [ASSAKQ, SOOQNR, OOQNRS]
PICXQI = [ASSAKQ, QXPICX, XPICXQ]
PIPIVL = [
    THECVY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    ECVYYP,
    CVYYPI,
    VYYPIP,
    YYPIPI,
    HECVYY,
    YPIPIV,
]
PSCTTG = [
    AKQXPI,
    XQIEFX,
    IEFXQG,
    EFXQGL,
    FXQGLR,
    GLRAHE,
    HEOCTH,
    OCTHBS,
    THBSKS,
    BSKSOK,
    KSOKPH,
    OKPHUO,
    PHUOJR,
    UOJRJH,
    JRJHIU,
    JHIUSO,
]
QGLRAH = [ASSAKQ, XPICXQ]
QRJJJV = [SBDJQR, BDJQRJ, DJQRJJ, JQRJJJ]
RAHEOC = [ASSAKQ, LRAHEO]
SELHBQ = [UGSELH, GSELHB]
SSAKQX = [IVLASS, VLASSA, LASSAK, ASSAKQ]
TSEMCG = [
    HBQNRX,
    BQNRXC,
    QNRXCH,
    NRXCHW,
    RXCHWD,
    XCHWDA,
    CHWDAF,
    HWDAFI,
    WDAFIK,
    SJMCBF,
    DAFIKJ,
    BFEGZU,
    FEGZUQ,
    GZUQEX,
    HECVYY,
    HECVYY,
    STSEMC,
    HECVYY,
    HECVYY,
    AFIKJP,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    EXLSXU,
    FIKJPU,
]
USPBWJ = [GYOUSP, QXPICX, YOUSPB, XPICXQ, OUSPBW]
UTYEKC = [XUJUTY, UJUTYE, JUTYEK]
XQVXOI = [
    HBQNRX,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    HECVYY,
    BFEGZU,
]
YGDSBD = [
    BSSUHB,
    SSUHBV,
    SUHBVW,
    UHBVWV,
    HBVWVU,
    BVWVUB,
    VWVUBY,
    WVUBYG,
    VUBYGD,
    UBYGDS,
    BYGDSB,
]
YIYWSK = [GTYIYW, TYIYWS]
YXBQFY = [IRYXBQ, RYXBQF]
YYLIUX = [NPYYLI, PYYLIU]


class GeckoLogStruct:
    def __init__(self, struct_):
        self.struct = struct_

    @property
    def version(self):
        return SCTTGC

    @property
    def begin(self):
        return CTTGCR

    @property
    def end(self):
        return TTGCRH

    @property
    def all_device_keys(self):
        return MPSCTT

    @property
    def user_demand_keys(self):
        return PSCTTG

    @property
    def accessors(self):
        return {
            ZCQBMJ: GeckoTempStructAccessor(self.struct, ZCQBMJ, CQBMJV, None),
            QBMJVH: GeckoByteStructAccessor(self.struct, QBMJVH, BMJVHF, None),
            MJVHFT: GeckoBoolStructAccessor(self.struct, MJVHFT, JVHFTH, VHFTHE, None),
            HFTHEC: GeckoEnumStructAccessor(
                self.struct, HFTHEC, FTHECV, None, PIPIVL, None, None, None
            ),
            IPIVLA: GeckoEnumStructAccessor(
                self.struct, IPIVLA, PIVLAS, None, SSAKQX, None, None, SAKQXP
            ),
            AKQXPI: GeckoEnumStructAccessor(
                self.struct, AKQXPI, KQXPIC, ICXQIE, PICXQI, None, CXQIEF, SAKQXP
            ),
            XQIEFX: GeckoEnumStructAccessor(
                self.struct, XQIEFX, KQXPIC, QIEFXQ, PICXQI, None, CXQIEF, SAKQXP
            ),
            IEFXQG: GeckoEnumStructAccessor(
                self.struct, IEFXQG, KQXPIC, CXQIEF, PICXQI, None, CXQIEF, SAKQXP
            ),
            EFXQGL: GeckoEnumStructAccessor(
                self.struct, EFXQGL, KQXPIC, VHFTHE, PICXQI, None, CXQIEF, SAKQXP
            ),
            FXQGLR: GeckoEnumStructAccessor(
                self.struct, FXQGLR, XQGLRA, ICXQIE, QGLRAH, None, QIEFXQ, SAKQXP
            ),
            GLRAHE: GeckoEnumStructAccessor(
                self.struct, GLRAHE, XQGLRA, AHEOCT, RAHEOC, None, QIEFXQ, SAKQXP
            ),
            HEOCTH: GeckoEnumStructAccessor(
                self.struct, HEOCTH, EOCTHB, None, RAHEOC, None, None, SAKQXP
            ),
            OCTHBS: GeckoEnumStructAccessor(
                self.struct, OCTHBS, CTHBSK, None, QGLRAH, None, None, SAKQXP
            ),
            THBSKS: GeckoEnumStructAccessor(
                self.struct, THBSKS, HBSKSO, None, RAHEOC, None, None, SAKQXP
            ),
            BSKSOK: GeckoEnumStructAccessor(
                self.struct, BSKSOK, SKSOKP, None, RAHEOC, None, None, SAKQXP
            ),
            KSOKPH: GeckoByteStructAccessor(self.struct, KSOKPH, SOKPHU, SAKQXP),
            OKPHUO: GeckoByteStructAccessor(self.struct, OKPHUO, KPHUOJ, SAKQXP),
            PHUOJR: GeckoByteStructAccessor(self.struct, PHUOJR, HUOJRJ, SAKQXP),
            UOJRJH: GeckoByteStructAccessor(self.struct, UOJRJH, OJRJHI, SAKQXP),
            JRJHIU: GeckoByteStructAccessor(self.struct, JRJHIU, RJHIUS, SAKQXP),
            JHIUSO: GeckoByteStructAccessor(self.struct, JHIUSO, HIUSOO, SAKQXP),
            IUSOOQ: GeckoEnumStructAccessor(
                self.struct, IUSOOQ, USOOQN, ICXQIE, OQNRSJ, None, CXQIEF, None
            ),
            QNRSJM: GeckoEnumStructAccessor(
                self.struct, QNRSJM, USOOQN, QIEFXQ, OQNRSJ, None, CXQIEF, None
            ),
            NRSJMC: GeckoEnumStructAccessor(
                self.struct, NRSJMC, USOOQN, CXQIEF, OQNRSJ, None, CXQIEF, None
            ),
            RSJMCB: GeckoEnumStructAccessor(
                self.struct, RSJMCB, USOOQN, VHFTHE, OQNRSJ, None, CXQIEF, None
            ),
            SJMCBF: GeckoEnumStructAccessor(
                self.struct, SJMCBF, JMCBFE, ICXQIE, MCBFEG, None, QIEFXQ, None
            ),
            CBFEGZ: GeckoEnumStructAccessor(
                self.struct, CBFEGZ, JMCBFE, AHEOCT, RAHEOC, None, QIEFXQ, None
            ),
            BFEGZU: GeckoEnumStructAccessor(
                self.struct, BFEGZU, JMCBFE, QIEFXQ, RAHEOC, None, QIEFXQ, None
            ),
            FEGZUQ: GeckoEnumStructAccessor(
                self.struct, FEGZUQ, JMCBFE, EGZUQE, RAHEOC, None, QIEFXQ, None
            ),
            GZUQEX: GeckoEnumStructAccessor(
                self.struct, GZUQEX, JMCBFE, CXQIEF, RAHEOC, None, QIEFXQ, None
            ),
            ZUQEXL: GeckoEnumStructAccessor(
                self.struct, ZUQEXL, JMCBFE, UQEXLS, RAHEOC, None, QIEFXQ, None
            ),
            QEXLSX: GeckoEnumStructAccessor(
                self.struct, QEXLSX, JMCBFE, VHFTHE, RAHEOC, None, QIEFXQ, None
            ),
            EXLSXU: GeckoEnumStructAccessor(
                self.struct, EXLSXU, JMCBFE, XLSXUJ, RAHEOC, None, QIEFXQ, None
            ),
            LSXUJU: GeckoEnumStructAccessor(
                self.struct, LSXUJU, SXUJUT, None, UTYEKC, None, None, SAKQXP
            ),
            TYEKCW: GeckoEnumStructAccessor(
                self.struct, TYEKCW, YEKCWA, None, UTYEKC, None, None, None
            ),
            EKCWAO: GeckoWordStructAccessor(self.struct, EKCWAO, KCWAON, None),
            CWAONP: GeckoWordStructAccessor(self.struct, CWAONP, WAONPY, SAKQXP),
            AONPYY: GeckoEnumStructAccessor(
                self.struct, AONPYY, ONPYYL, ICXQIE, YYLIUX, None, QIEFXQ, SAKQXP
            ),
            YLIUXF: GeckoEnumStructAccessor(
                self.struct, YLIUXF, LIUXFE, None, FJTACC, None, None, SAKQXP
            ),
            JTACCP: GeckoTimeStructAccessor(self.struct, JTACCP, TACCPQ, SAKQXP),
            ACCPQI: GeckoByteStructAccessor(self.struct, ACCPQI, CCPQIP, SAKQXP),
            CPQIPO: GeckoEnumStructAccessor(
                self.struct, CPQIPO, ONPYYL, QIEFXQ, YYLIUX, None, QIEFXQ, SAKQXP
            ),
            PQIPOU: GeckoEnumStructAccessor(
                self.struct, PQIPOU, QIPOUY, None, FJTACC, None, None, SAKQXP
            ),
            IPOUYN: GeckoTimeStructAccessor(self.struct, IPOUYN, POUYNQ, SAKQXP),
            OUYNQJ: GeckoByteStructAccessor(self.struct, OUYNQJ, UYNQJY, SAKQXP),
            YNQJYM: GeckoByteStructAccessor(self.struct, YNQJYM, NQJYMO, SAKQXP),
            QJYMOU: GeckoByteStructAccessor(self.struct, QJYMOU, JYMOUN, SAKQXP),
            YMOUNB: GeckoEnumStructAccessor(
                self.struct, YMOUNB, ONPYYL, AHEOCT, YYLIUX, None, QIEFXQ, SAKQXP
            ),
            MOUNBL: GeckoEnumStructAccessor(
                self.struct, MOUNBL, OUNBLK, None, FJTACC, None, None, SAKQXP
            ),
            UNBLKX: GeckoTimeStructAccessor(self.struct, UNBLKX, NBLKXS, SAKQXP),
            BLKXSJ: GeckoBoolStructAccessor(self.struct, BLKXSJ, LKXSJW, ICXQIE, None),
            KXSJWM: GeckoBoolStructAccessor(self.struct, KXSJWM, LKXSJW, QIEFXQ, None),
            XSJWMN: GeckoBoolStructAccessor(self.struct, XSJWMN, LKXSJW, EGZUQE, None),
            SJWMNZ: GeckoBoolStructAccessor(self.struct, SJWMNZ, LKXSJW, CXQIEF, None),
            JWMNZM: GeckoBoolStructAccessor(self.struct, JWMNZM, LKXSJW, UQEXLS, None),
            WMNZMJ: GeckoBoolStructAccessor(self.struct, WMNZMJ, JVHFTH, QIEFXQ, None),
            MNZMJI: GeckoBoolStructAccessor(self.struct, MNZMJI, NZMJIG, EGZUQE, None),
            ZMJIGY: GeckoBoolStructAccessor(self.struct, ZMJIGY, NZMJIG, UQEXLS, None),
            MJIGYO: GeckoBoolStructAccessor(self.struct, MJIGYO, NZMJIG, VHFTHE, None),
            JIGYOU: GeckoEnumStructAccessor(
                self.struct, JIGYOU, IGYOUS, None, USPBWJ, None, None, None
            ),
            SPBWJY: GeckoBoolStructAccessor(self.struct, SPBWJY, PBWJYK, UQEXLS, None),
            BWJYKL: GeckoBoolStructAccessor(self.struct, BWJYKL, PBWJYK, VHFTHE, None),
            WJYKLG: GeckoWordStructAccessor(self.struct, WJYKLG, JYKLGQ, None),
            YKLGQP: GeckoBoolStructAccessor(self.struct, YKLGQP, JVHFTH, AHEOCT, None),
            KLGQPL: GeckoTempStructAccessor(self.struct, KLGQPL, LGQPLS, None),
            GQPLSP: GeckoTempStructAccessor(self.struct, GQPLSP, QPLSPF, None),
            PLSPFT: GeckoEnumStructAccessor(
                self.struct, PLSPFT, JMCBFE, UQEXLS, LSPFTS, None, CXQIEF, None
            ),
            SPFTSI: GeckoBoolStructAccessor(self.struct, SPFTSI, PFTSIF, QIEFXQ, None),
            FTSIFJ: GeckoBoolStructAccessor(self.struct, FTSIFJ, PFTSIF, VHFTHE, None),
            TSIFJB: GeckoBoolStructAccessor(self.struct, TSIFJB, SIFJBI, QIEFXQ, None),
            IFJBIA: GeckoBoolStructAccessor(self.struct, IFJBIA, FJBIAM, AHEOCT, None),
            JBIAMJ: GeckoBoolStructAccessor(
                self.struct, JBIAMJ, FJBIAM, QIEFXQ, SAKQXP
            ),
            BIAMJM: GeckoBoolStructAccessor(self.struct, BIAMJM, IAMJMA, AHEOCT, None),
            AMJMAO: GeckoByteStructAccessor(self.struct, AMJMAO, MJMAOA, None),
            JMAOAW: GeckoBoolStructAccessor(self.struct, JMAOAW, MAOAWB, ICXQIE, None),
            AOAWBS: GeckoBoolStructAccessor(self.struct, AOAWBS, OAWBSI, ICXQIE, None),
            AWBSIR: GeckoBoolStructAccessor(self.struct, AWBSIR, OAWBSI, CXQIEF, None),
            WBSIRY: GeckoBoolStructAccessor(self.struct, WBSIRY, OAWBSI, UQEXLS, None),
            BSIRYX: GeckoBoolStructAccessor(self.struct, BSIRYX, OAWBSI, VHFTHE, None),
            SIRYXB: GeckoEnumStructAccessor(
                self.struct, SIRYXB, OAWBSI, XLSXUJ, YXBQFY, None, QIEFXQ, SAKQXP
            ),
            XBQFYL: GeckoEnumStructAccessor(
                self.struct, XBQFYL, BQFYLJ, None, FWRKIN, None, None, None
            ),
            WRKINE: GeckoBoolStructAccessor(self.struct, WRKINE, RKINEJ, ICXQIE, None),
            KINEJN: GeckoBoolStructAccessor(self.struct, KINEJN, RKINEJ, AHEOCT, None),
            INEJNI: GeckoBoolStructAccessor(self.struct, INEJNI, NEJNIB, ICXQIE, None),
            EJNIBX: GeckoBoolStructAccessor(self.struct, EJNIBX, NEJNIB, AHEOCT, None),
            JNIBXY: GeckoBoolStructAccessor(self.struct, JNIBXY, NIBXYB, ICXQIE, None),
            IBXYBQ: GeckoBoolStructAccessor(self.struct, IBXYBQ, JVHFTH, EGZUQE, None),
            BXYBQS: GeckoBoolStructAccessor(self.struct, BXYBQS, NZMJIG, ICXQIE, None),
            XYBQSN: GeckoBoolStructAccessor(self.struct, XYBQSN, NZMJIG, AHEOCT, None),
            YBQSNQ: GeckoBoolStructAccessor(self.struct, YBQSNQ, BQSNQL, QIEFXQ, None),
            QSNQLN: GeckoBoolStructAccessor(self.struct, QSNQLN, BQSNQL, EGZUQE, None),
            SNQLNM: GeckoBoolStructAccessor(self.struct, SNQLNM, RKINEJ, EGZUQE, None),
            NQLNMH: GeckoBoolStructAccessor(self.struct, NQLNMH, RKINEJ, CXQIEF, None),
            QLNMHX: GeckoBoolStructAccessor(self.struct, QLNMHX, RKINEJ, UQEXLS, None),
            LNMHXE: GeckoBoolStructAccessor(self.struct, LNMHXE, RKINEJ, VHFTHE, None),
            NMHXEK: GeckoBoolStructAccessor(self.struct, NMHXEK, NEJNIB, EGZUQE, None),
            MHXEKV: GeckoBoolStructAccessor(self.struct, MHXEKV, NEJNIB, CXQIEF, None),
            HXEKVK: GeckoBoolStructAccessor(self.struct, HXEKVK, NEJNIB, UQEXLS, None),
            XEKVKZ: GeckoBoolStructAccessor(self.struct, XEKVKZ, NEJNIB, VHFTHE, None),
            EKVKZI: GeckoBoolStructAccessor(self.struct, EKVKZI, KVKZIL, QIEFXQ, None),
            VKZILX: GeckoBoolStructAccessor(self.struct, VKZILX, KVKZIL, EGZUQE, None),
            KZILXW: GeckoBoolStructAccessor(self.struct, KZILXW, IAMJMA, EGZUQE, None),
            ZILXWA: GeckoBoolStructAccessor(self.struct, ZILXWA, PBWJYK, ICXQIE, None),
            ILXWAJ: GeckoBoolStructAccessor(self.struct, ILXWAJ, PBWJYK, AHEOCT, None),
            LXWAJV: GeckoBoolStructAccessor(self.struct, LXWAJV, XWAJVD, ICXQIE, None),
            WAJVDQ: GeckoBoolStructAccessor(self.struct, WAJVDQ, SIFJBI, ICXQIE, None),
            AJVDQL: GeckoBoolStructAccessor(self.struct, AJVDQL, BQSNQL, ICXQIE, None),
            JVDQLA: GeckoBoolStructAccessor(self.struct, JVDQLA, BQSNQL, AHEOCT, None),
            VDQLAI: GeckoBoolStructAccessor(self.struct, VDQLAI, BQSNQL, CXQIEF, None),
            DQLAII: GeckoBoolStructAccessor(self.struct, DQLAII, QLAIID, ICXQIE, None),
            LAIIDN: GeckoEnumStructAccessor(
                self.struct, LAIIDN, QLAIID, AHEOCT, AIIDNI, None, CXQIEF, None
            ),
            IIDNIB: GeckoBoolStructAccessor(self.struct, IIDNIB, QLAIID, EGZUQE, None),
            IDNIBX: GeckoWordStructAccessor(self.struct, IDNIBX, DNIBXT, None),
            NIBXTI: GeckoBoolStructAccessor(self.struct, NIBXTI, IBXTIA, ICXQIE, None),
            BXTIAC: GeckoBoolStructAccessor(self.struct, BXTIAC, XTIACQ, ICXQIE, None),
            TIACQF: GeckoBoolStructAccessor(self.struct, TIACQF, XTIACQ, AHEOCT, None),
            IACQFF: GeckoBoolStructAccessor(self.struct, IACQFF, KVKZIL, ICXQIE, None),
            ACQFFT: GeckoBoolStructAccessor(self.struct, ACQFFT, KVKZIL, AHEOCT, None),
            CQFFTT: GeckoBoolStructAccessor(self.struct, CQFFTT, KVKZIL, CXQIEF, None),
            QFFTTI: GeckoWordStructAccessor(self.struct, QFFTTI, FFTTID, None),
            FTTIDU: GeckoByteStructAccessor(self.struct, FTTIDU, TTIDUB, None),
            TIDUBS: GeckoByteStructAccessor(self.struct, TIDUBS, IDUBSS, None),
            DUBSSU: GeckoEnumStructAccessor(
                self.struct, DUBSSU, UBSSUH, None, YGDSBD, None, None, None
            ),
            GDSBDJ: GeckoEnumStructAccessor(
                self.struct, GDSBDJ, DSBDJQ, ICXQIE, QRJJJV, None, CXQIEF, None
            ),
            RJJJVY: GeckoEnumStructAccessor(
                self.struct, RJJJVY, DSBDJQ, QIEFXQ, JVYFCR, None, QIEFXQ, None
            ),
            VYFCRT: GeckoWordStructAccessor(self.struct, VYFCRT, YFCRTF, None),
            FCRTFM: GeckoByteStructAccessor(self.struct, FCRTFM, CRTFMN, None),
            RTFMNH: GeckoByteStructAccessor(self.struct, RTFMNH, TFMNHT, None),
            FMNHTB: GeckoByteStructAccessor(self.struct, FMNHTB, MNHTBJ, None),
            NHTBJE: GeckoByteStructAccessor(self.struct, NHTBJE, HTBJEU, None),
            TBJEUT: GeckoWordStructAccessor(self.struct, TBJEUT, BJEUTO, None),
            JEUTOP: GeckoByteStructAccessor(self.struct, JEUTOP, EUTOPH, None),
            UTOPHU: GeckoByteStructAccessor(self.struct, UTOPHU, TOPHUG, None),
            OPHUGT: GeckoWordStructAccessor(self.struct, OPHUGT, PHUGTY, None),
            HUGTYI: GeckoEnumStructAccessor(
                self.struct, HUGTYI, UGTYIY, None, YIYWSK, None, QIEFXQ, SAKQXP
            ),
            IYWSKW: GeckoEnumStructAccessor(
                self.struct, IYWSKW, YWSKWI, None, NXNKML, None, None, None
            ),
            XNKMLO: GeckoWordStructAccessor(self.struct, XNKMLO, NKMLOI, None),
            KMLOIJ: GeckoByteStructAccessor(self.struct, KMLOIJ, MLOIJU, None),
            LOIJUG: GeckoByteStructAccessor(self.struct, LOIJUG, OIJUGS, None),
            IJUGSE: GeckoEnumStructAccessor(
                self.struct, IJUGSE, JUGSEL, None, SELHBQ, None, None, SAKQXP
            ),
            ELHBQN: GeckoEnumStructAccessor(
                self.struct, ELHBQN, LHBQNR, None, IKJPUN, None, None, SAKQXP
            ),
            KJPUNR: GeckoEnumStructAccessor(
                self.struct, KJPUNR, JPUNRJ, None, IKJPUN, None, None, SAKQXP
            ),
            PUNRJZ: GeckoEnumStructAccessor(
                self.struct, PUNRJZ, UNRJZT, None, IKJPUN, None, None, SAKQXP
            ),
            NRJZTA: GeckoEnumStructAccessor(
                self.struct, NRJZTA, RJZTAT, None, IKJPUN, None, None, SAKQXP
            ),
            JZTATD: GeckoEnumStructAccessor(
                self.struct, JZTATD, ZTATDZ, None, IKJPUN, None, None, SAKQXP
            ),
            TATDZX: GeckoEnumStructAccessor(
                self.struct, TATDZX, ATDZXN, None, DZXNQT, None, None, SAKQXP
            ),
            ZXNQTM: GeckoByteStructAccessor(self.struct, ZXNQTM, XNQTMF, SAKQXP),
            NQTMFZ: GeckoByteStructAccessor(self.struct, NQTMFZ, QTMFZD, SAKQXP),
            TMFZDG: GeckoByteStructAccessor(self.struct, TMFZDG, MFZDGK, SAKQXP),
            FZDGKE: GeckoByteStructAccessor(self.struct, FZDGKE, ZDGKEA, SAKQXP),
            DGKEAK: GeckoByteStructAccessor(self.struct, DGKEAK, GKEAKS, SAKQXP),
            KEAKST: GeckoByteStructAccessor(self.struct, KEAKST, EAKSTS, SAKQXP),
            AKSTSE: GeckoEnumStructAccessor(
                self.struct, AKSTSE, KSTSEM, None, TSEMCG, None, None, SAKQXP
            ),
            SEMCGE: GeckoEnumStructAccessor(
                self.struct, SEMCGE, ATDZXN, None, TSEMCG, None, None, SAKQXP
            ),
            EMCGET: GeckoEnumStructAccessor(
                self.struct, EMCGET, XNQTMF, None, TSEMCG, None, None, SAKQXP
            ),
            MCGETI: GeckoEnumStructAccessor(
                self.struct, MCGETI, QTMFZD, None, TSEMCG, None, None, SAKQXP
            ),
            CGETIX: GeckoEnumStructAccessor(
                self.struct, CGETIX, MFZDGK, None, TSEMCG, None, None, SAKQXP
            ),
            GETIXQ: GeckoEnumStructAccessor(
                self.struct, GETIXQ, ZDGKEA, None, TSEMCG, None, None, SAKQXP
            ),
            ETIXQV: GeckoEnumStructAccessor(
                self.struct, ETIXQV, GKEAKS, None, TSEMCG, None, None, SAKQXP
            ),
            TIXQVX: GeckoEnumStructAccessor(
                self.struct, TIXQVX, IXQVXO, None, XQVXOI, None, None, SAKQXP
            ),
            QVXOIH: GeckoEnumStructAccessor(
                self.struct, QVXOIH, EAKSTS, None, XQVXOI, None, None, SAKQXP
            ),
            VXOIHB: GeckoByteStructAccessor(self.struct, VXOIHB, XOIHBX, SAKQXP),
            OIHBXI: GeckoByteStructAccessor(self.struct, OIHBXI, IHBXIB, SAKQXP),
            HBXIBH: GeckoByteStructAccessor(self.struct, HBXIBH, BXIBHZ, SAKQXP),
            XIBHZV: GeckoByteStructAccessor(self.struct, XIBHZV, IBHZVO, SAKQXP),
            BHZVOA: GeckoByteStructAccessor(self.struct, BHZVOA, HZVOAC, SAKQXP),
            ZVOACM: GeckoByteStructAccessor(self.struct, ZVOACM, VOACMC, SAKQXP),
            OACMCV: GeckoByteStructAccessor(self.struct, OACMCV, ACMCVD, SAKQXP),
            CMCVDS: GeckoByteStructAccessor(self.struct, CMCVDS, MCVDSS, SAKQXP),
            CVDSSR: GeckoByteStructAccessor(self.struct, CVDSSR, VDSSRU, SAKQXP),
            VEMVCY: GeckoBoolStructAccessor(self.struct, VEMVCY, JVHFTH, UQEXLS, None),
            EMVCYW: GeckoByteStructAccessor(self.struct, EMVCYW, MVCYWO, SAKQXP),
            VCYWON: GeckoByteStructAccessor(self.struct, VCYWON, CYWONF, SAKQXP),
            YWONFZ: GeckoByteStructAccessor(self.struct, YWONFZ, WONFZC, SAKQXP),
            ONFZCZ: GeckoEnumStructAccessor(
                self.struct, ONFZCZ, NFZCZO, None, LSIPMD, None, None, SAKQXP
            ),
            SIPMDM: GeckoBoolStructAccessor(
                self.struct, SIPMDM, IPMDMP, ICXQIE, SAKQXP
            ),
            PMDMPS: GeckoBoolStructAccessor(self.struct, PMDMPS, JUGSEL, ICXQIE, None),
        }



""" GeckoSimulator class """

import logging
import os
import glob
import random
import importlib
import struct
from ..const import GeckoConstants

from .shared_command import GeckoCmd
from ..driver import (
    GeckoHelloProtocolHandler,
    GeckoPacketProtocolHandler,
    GeckoPingProtocolHandler,
    GeckoVersionProtocolHandler,
    GeckoGetChannelProtocolHandler,
    GeckoConfigFileProtocolHandler,
    GeckoStatusBlockProtocolHandler,
    GeckoPartialStatusBlockProtocolHandler,
    GeckoWatercareProtocolHandler,
    GeckoUpdateFirmwareProtocolHandler,
    GeckoRemindersProtocolHandler,
    GeckoRFErrProtocolHandler,
    GeckoPackCommandProtocolHandler,
    GeckoStructure,
    GeckoUdpSocket,
)
from .snapshot import GeckoSnapshot
from .. import VERSION

_LOGGER = logging.getLogger(__name__)


class GeckoSimulator(GeckoCmd):
    """GeckoSimulator is a server application to aid with investigating
    Gecko protocol"""

    _STATUS_BLOCK_SEGMENT_SIZE = 39

    def __init__(self, first_commands=None):

        self._socket = GeckoUdpSocket()
        self._install_standard_handlers()
        self.structure = GeckoStructure(self._on_set_value)
        self.snapshot = None
        self._reliability = 1.0
        self._do_rferr = False
        self._send_structure_change = False
        self._clients = []
        random.seed()

        super().__init__(first_commands)

        self.intro = (
            "Welcome to the Gecko simulator. Type help or ? to list commands.\n"
        )
        self.prompt = "(GeckoSim) "
        try:
            import readline

            readline.set_completer_delims(" \r\n")
        except ImportError:
            pass

    def __exit__(self, *args):
        self._socket.close()
        self._socket = None

    def do_about(self, _arg):
        """Display information about this client program and support library : about"""
        print("")
        print(
            "GeckoSimulator: A python program using GeckoLib library to simulate Gecko"
            " enabled devices with in.touch2 communication modules"
        )
        print("Library version v{0}".format(VERSION))

    def do_start(self, args):
        """Start the configured simulator : start"""
        if self._socket.isopen:
            print("Simulator is already started")
            return
        self._socket.open()
        self._socket.enable_broadcast()
        self._socket.bind()

    def _complete_path(self, path):
        if os.path.isdir(path):
            return glob.glob(os.path.join(path, "*"))
        return glob.glob(path + "*")

    def do_parse(self, args):
        """Parse logfiles to extract snapshots to the ./snapshot directory. Will
        overwrite identically named snapshot files if present : parse <logfile>"""
        for snapshot in GeckoSnapshot.parse_log_file(args):
            snapshot.save("snapshots")
            print(f"Saved snapshot snapshots/{snapshot.filename}")

    def do_reliability(self, args):
        """Set simulator reliability factor. Reliability is a measure of how likely
        the simulator will respond to an incoming message. Reliability of 1.0 (default)
        means the simulator will always respond, whereas 0.0 means it will never
        respond. This does not take into account messages that actually don't get
        recieved : reliability <factor> where <factor> is a float between 0.0 and
        1.0."""
        if args == "":
            print(f"Current reliability is {self._reliability}")
            return
        self._reliability = min(1.0, max(0.0, float(args)))

    def do_rferr(self, args):
        """Set the simulator to response with RFERR if the parameter is True"""
        self._do_rferr = args.lower() == "true"
        print(f"RFERR mode set to {self._do_rferr}")

    def do_get(self, arg):
        """Get the value of the specified spa pack structure element : get <Element>"""
        try:
            print("{0} = {1}".format(arg, self.structure.accessors[arg].value))
        except Exception:  # pylint: disable=broad-except
            _LOGGER.exception("Exception getting '%s'", arg)

    def do_set(self, arg):
        """Set the value of the specified spa pack structure
        element : set <Element>=<value>"""
        self._send_structure_change = True
        try:
            key, val = arg.split("=")
            self.structure.accessors[key].value = val
        except Exception:  # pylint: disable=broad-except
            _LOGGER.exception("Exception handling 'set %s'", arg)
        finally:
            self._send_structure_change = False

    def do_accessors(self, _arg):
        """Display the data from the accessors : accessors"""
        print("Accessors")
        print("=========")
        print("")
        for key in self.structure.accessors:
            print("   {0}: {1}".format(key, self.structure.accessors[key].value))
        print("")

    def complete_parse(self, text, line, start_idx, end_idx):
        return self._complete_path(text)

    def do_load(self, args):
        """Load a snapshot : load <snapshot>"""
        snapshots = GeckoSnapshot.parse_log_file(args)
        if len(snapshots) == 1:
            self.set_snapshot(snapshots[0])
            return
        print(
            f"{args} contains {len(snapshots)} snapshots. Please use the"
            f" `parse` command to break it apart"
        )

    def complete_load(self, text, line, start_idx, end_idx):
        return self._complete_path(text)

    def set_snapshot(self, snapshot):
        self.snapshot = snapshot
        self.structure.replace_status_block_segment(0, self.snapshot.bytes)

        try:
            # Attempt to get config and log classes
            plateform_key = self.snapshot.packtype.lower()

            pack_module_name = f"geckolib.driver.packs.{plateform_key}"
            try:
                GeckoPack = importlib.import_module(pack_module_name).GeckoPack
                self.pack_class = GeckoPack(self.structure)
                self.pack_type = self.pack_class.type
            except ModuleNotFoundError:
                raise Exception(
                    GeckoConstants.EXCEPTION_MESSAGE_NO_SPA_PACK.format(
                        self.snapshot.packtype
                    )
                )

            config_module_name = f"geckolib.driver.packs.{plateform_key}-cfg-{self.snapshot.config_version}"
            try:
                GeckoConfigStruct = importlib.import_module(
                    config_module_name
                ).GeckoConfigStruct
                self.config_class = GeckoConfigStruct(self.structure)
            except ModuleNotFoundError:
                raise Exception(
                    f"Cannot find GeckoConfigStruct module for {self.snapshot.packtype} v{self.snapshot.config_version}"
                )

            log_module_name = (
                f"geckolib.driver.packs.{plateform_key}-log-{self.snapshot.log_version}"
            )
            try:
                GeckoLogStruct = importlib.import_module(log_module_name).GeckoLogStruct
                self.log_class = GeckoLogStruct(self.structure)
            except ModuleNotFoundError:
                raise Exception(
                    f"Cannot find GeckoLogStruct module for {self.snapshot.packtype} v{self.snapshot.log_version}"
                )

            self.structure.build_accessors(self.config_class, self.log_class)
            for accessor in self.structure.accessors.values():
                accessor.set_read_write("ALL")

        except:  # noqa
            _LOGGER.exception("Exception during snapshot load")

    def _should_ignore(self, handler, sender, respect_rferr=True):
        if respect_rferr and self._do_rferr:
            self._socket.queue_send(
                GeckoRFErrProtocolHandler.response(parms=sender),
                sender,
            )
            # Always ignore responses because we've already replied with RFERR
            return True

        should_ignore = random.random() > self._reliability
        if should_ignore:
            print(f"Unreliable simulator ignoring request for {handler} from {sender}")
        return should_ignore

    def _install_standard_handlers(self):
        """All simulators needs to have some basic functionality such
        as discovery, error handling et al"""
        self._hello_handler = GeckoHelloProtocolHandler.response(
            b"SPA01:02:03:04:05:06", "Udp Test Spa", on_handled=self._on_hello
        )

        self._socket.add_receive_handler(self._hello_handler)
        self._socket.add_receive_handler(
            GeckoPacketProtocolHandler(socket=self._socket)
        )
        self._socket.add_receive_handler(
            GeckoPingProtocolHandler(on_handled=self._on_ping)
        ),
        self._socket.add_receive_handler(
            GeckoVersionProtocolHandler(on_handled=self._on_version)
        )
        self._socket.add_receive_handler(
            GeckoGetChannelProtocolHandler(on_handled=self._on_get_channel)
        )
        self._socket.add_receive_handler(
            GeckoConfigFileProtocolHandler(on_handled=self._on_config_file)
        )
        self._socket.add_receive_handler(
            GeckoStatusBlockProtocolHandler(on_handled=self._on_status_block)
        )
        self._socket.add_receive_handler(
            GeckoWatercareProtocolHandler(on_handled=self._on_watercare)
        )
        self._socket.add_receive_handler(
            GeckoUpdateFirmwareProtocolHandler(on_handled=self._on_update_firmware)
        )
        self._socket.add_receive_handler(
            GeckoRemindersProtocolHandler(on_handled=self._on_get_reminders)
        )
        self._socket.add_receive_handler(
            GeckoPackCommandProtocolHandler(on_handled=self._on_pack_command)
        )

    def _on_hello(self, handler: GeckoHelloProtocolHandler, sender):
        if handler.was_broadcast_discovery:
            if self._should_ignore(handler, sender, False):
                return
            self._socket.queue_send(self._hello_handler, sender)
        elif handler.client_identifier is not None:
            # We're not fussy, we'll chat to anyone!
            pass

    def _on_ping(self, handler: GeckoPingProtocolHandler, sender):
        if self._should_ignore(handler, sender):
            return
        self._socket.queue_send(
            GeckoPingProtocolHandler.response(parms=sender),
            sender,
        )
        if sender not in self._clients:
            self._clients.append(sender)

    def _on_version(self, handler: GeckoVersionProtocolHandler, sender):
        if self._should_ignore(handler, sender):
            return
        self._socket.queue_send(
            GeckoVersionProtocolHandler.response(
                self.snapshot.intouch_EN,
                self.snapshot.intouch_CO,
                parms=sender,
            ),
            sender,
        )

    def _on_get_channel(self, handler: GeckoGetChannelProtocolHandler, sender):
        if self._should_ignore(handler, sender):
            return
        self._socket.queue_send(
            GeckoGetChannelProtocolHandler.response(10, 33, parms=sender),
            sender,
        )

    def _on_config_file(self, handler: GeckoConfigFileProtocolHandler, sender):
        if self._should_ignore(handler, sender):
            return
        self._socket.queue_send(
            GeckoConfigFileProtocolHandler.response(
                self.snapshot.packtype,
                self.snapshot.config_version,
                self.snapshot.log_version,
                parms=sender,
            ),
            sender,
        )

    def _on_watercare(self, handler: GeckoWatercareProtocolHandler, sender):
        if self._should_ignore(handler, sender):
            return
        if handler.schedule:
            self._socket.queue_send(
                GeckoWatercareProtocolHandler.giveschedule(parms=sender), sender
            )
        else:
            self._socket.queue_send(
                GeckoWatercareProtocolHandler.response(1, parms=sender), sender
            )

    def _on_update_firmware(self, handler: GeckoUpdateFirmwareProtocolHandler, sender):
        if self._should_ignore(handler, sender):
            return
        self._socket.queue_send(
            GeckoUpdateFirmwareProtocolHandler.response(parms=sender), sender
        )

    def _on_status_block(self, handler: GeckoStatusBlockProtocolHandler, sender):
        if self._should_ignore(handler, sender):
            return
        for idx, start in enumerate(
            range(
                handler.start,
                handler.start + handler.length,
                self._STATUS_BLOCK_SEGMENT_SIZE,
            )
        ):
            length = min(
                self._STATUS_BLOCK_SEGMENT_SIZE,
                len(self.structure.status_block) - start,
            )
            next = (idx + 1) % ((handler.length // self._STATUS_BLOCK_SEGMENT_SIZE) + 1)
            if self._should_ignore(handler, sender, False):
                continue
            self._socket.queue_send(
                GeckoStatusBlockProtocolHandler.response(
                    idx,
                    next,
                    self.structure.status_block[start : start + length],
                    parms=sender,
                ),
                sender,
            )

    def _on_get_reminders(self, handler: GeckoRemindersProtocolHandler, sender):
        if self._should_ignore(handler, sender):
            return
        self._socket.queue_send(
            GeckoRemindersProtocolHandler.response(parms=sender), sender
        )

    def _on_pack_command(self, handler: GeckoPackCommandProtocolHandler, sender):
        if self._should_ignore(handler, sender):
            return
        self._socket.queue_send(
            GeckoPackCommandProtocolHandler.response(parms=sender), sender
        )
        if handler.is_key_press:
            print(f"Pressed a key ({handler.keycode})")
        elif handler.is_set_value:
            print(f"Set a value ({handler.position} = {handler.new_data})")

    def _on_set_value(self, pos, length, newvalue):
        print(f"Simulator: Set value @{pos} len {length} to {newvalue}")
        change = None
        if length == 1:
            change = (pos, struct.pack(">B", newvalue))
        elif length == 2:
            change = (pos, struct.pack(">H", newvalue))
        else:
            print("**** UNHANDLED SET SIZE ****")
            return

        self.structure.replace_status_block_segment(change[0], change[1])

        if self._send_structure_change:
            for client in self._clients:
                self._socket.queue_send(
                    GeckoPartialStatusBlockProtocolHandler.report_changes(
                        self._socket, [change], parms=client
                    ),
                    client,
                )



""" Gecko Watercare """

import logging
from .base import GeckoAutomationFacadeBase

from ..driver import GeckoWatercareProtocolHandler
from ..const import GeckoConstants

_LOGGER = logging.getLogger(__name__)


class GeckoWaterCare(GeckoAutomationFacadeBase):
    """Watercare manangement class"""

    def __init__(self, facade):
        super().__init__(facade, "WaterCare", "WATERCARE")
        self.active_mode = None
        self._water_care_handler = None

    @property
    def mode(self):
        """Return the active water care mode"""
        return self.active_mode

    @property
    def modes(self):
        """Return all the possible water care modes"""
        return GeckoConstants.WATERCARE_MODE_STRING

    def set_mode(self, new_mode):
        """Set the active watercare mode to new_mode.
        new_mode can be a string, in which case the value must be a member of
        GeckoConstants.WATERCARE_MODE_STRING, or it can be an integer from
        GeckoConstants.WATERCARE_MODE
        """
        if isinstance(new_mode, str):
            new_mode = GeckoConstants.WATERCARE_MODE_STRING.index(new_mode)
        self._spa.queue_send(
            GeckoWatercareProtocolHandler.set(
                self._spa.get_and_increment_sequence_counter(),
                new_mode,
                parms=self._spa.sendparms,
            ),
            self._spa.sendparms,
        )
        if new_mode != self.active_mode:
            self.active_mode = new_mode

    async def async_set_mode(self, new_mode):
        """Set the active watercare mode to new_mode.
        new_mode can be a string, in which case the value must be a member of
        GeckoConstants.WATERCARE_MODE_STRING, or it can be an integer from
        GeckoConstants.WATERCARE_MODE
        """
        if isinstance(new_mode, str):
            new_mode = GeckoConstants.WATERCARE_MODE_STRING.index(new_mode)
        await self._spa.async_set_watercare(new_mode)
        self.change_watercare_mode(new_mode)

    def _on_watercare(self, handler, sender):
        if self.active_mode != handler.mode:
            old_mode = self.active_mode
            self.active_mode = handler.mode
            self._on_change(self, old_mode, self.active_mode)
        self._water_care_handler = None

    def update(self):
        if self._water_care_handler is not None:
            return

        self._water_care_handler = GeckoWatercareProtocolHandler.request(
            self._spa.get_and_increment_sequence_counter(),
            on_handled=self._on_watercare,
            parms=self._spa.sendparms,
        )
        self._spa.add_receive_handler(self._water_care_handler)
        self._spa.queue_send(self._water_care_handler, self._spa.sendparms)

    def change_watercare_mode(self, new_mode):
        if self.active_mode != new_mode:
            old_mode = self.active_mode
            self.active_mode = new_mode
            self._on_change(self, old_mode, self.active_mode)

    def __str__(self):
        if self.active_mode is None:
            return f"{self.name}: Waiting..."
        if self.active_mode > len(GeckoConstants.WATERCARE_MODE_STRING):
            return f"Unknown Water care mode (index:{self.active_mode}"
        return f"{self.name}: {GeckoConstants.WATERCARE_MODE_STRING[self.active_mode]}"

    @property
    def monitor(self):
        if self.active_mode is None:
            return "WC: ?"
        return f"WC: {self.active_mode}"



"""Class to manage the clienting of a Gecko in.touch2 enabled device """
from __future__ import annotations

from abc import ABC, abstractmethod
import asyncio
from datetime import datetime


from .automation import GeckoAsyncFacade, GeckoAutomationBase, GeckoButton
from .async_locator import GeckoAsyncLocator
from .async_spa import GeckoAsyncSpa
from .async_spa_descriptor import GeckoAsyncSpaDescriptor
from .async_tasks import AsyncTasks
from .const import GeckoConstants
from .spa_events import GeckoSpaEvent
from .spa_state import GeckoSpaState

import logging
from typing import Optional, List

_LOGGER = logging.getLogger(__name__)


class GeckoAsyncSpaMan(ABC, AsyncTasks):
    """GeckoAsyncSpaMan class manages the lifetime of a connection to a spa

    This class is deliberately an abstract because you must provide your own
    implementation to manage the essential events that are required during operation
    """

    class ReconnectButton(GeckoButton):
        """Perform reconnect of the spa"""

        def __init__(self, spaman: GeckoAsyncSpaMan) -> None:
            super().__init__(
                spaman.unique_id, "Reconnect", spaman.spa_name, "RECONNECT"
            )
            self._spaman = spaman

        async def async_press(self):
            await self._spaman.async_reset()

    class PingSensor(GeckoAutomationBase):
        """Sensor with the last ping time, or None"""

        def __init__(self, spaman: GeckoAsyncSpaMan) -> None:
            super().__init__(spaman.unique_id, "Last Ping", spaman.spa_name, "PING")
            self._spaman: GeckoAsyncSpaMan = spaman
            assert self._spaman._spa is not None
            self._spaman._spa.watch(self._on_spa_change)
            self._last_ping_at: Optional[datetime] = None

        @property
        def state(self):
            """The state of the sensor"""
            return self._last_ping_at

        @property
        def unit_of_measurement(self):
            """The unit of measurement for the sensor, or None"""
            return None

        @property
        def device_class(self):
            return "timestamp"

        def _on_spa_change(self, *args):
            if self._last_ping_at == self._spaman._spa.last_ping_at:
                return
            self._last_ping_at = self._spaman._spa.last_ping_at
            self._on_change()

        def __repr__(self):
            return f"{self.name}: {self.state}"

    class StatusSensor(GeckoAutomationBase):
        """Sensor with the current state"""

        def __init__(self, spaman: GeckoAsyncSpaMan) -> None:
            super().__init__(spaman.unique_id, "Status", spaman.spa_name, "STATUS")
            self._spaman: GeckoAsyncSpaMan = spaman
            self._state = "Unknown"
            self._last_state = GeckoSpaState.IDLE
            self._last_event = GeckoSpaEvent.SPA_MAN_ENTER

        @property
        def state(self):
            """The state of the sensor"""
            return self._state

        @property
        def unit_of_measurement(self):
            """The unit of measurement for the sensor, or None"""
            return None

        @property
        def device_class(self):
            return "string"

        @property
        def last_event(self) -> GeckoSpaEvent:
            """Return the last event the sensor was notified of"""
            return self._last_event

        @property
        def spa_state(self) -> GeckoSpaState:
            """Returns the state of the spa at the time of the last event"""
            return self._last_state

        def on_event(self, event: GeckoSpaEvent) -> None:
            self._last_event = event
            self._last_state = self._spaman.spa_state

            if self.spa_state == GeckoSpaState.CONNECTED:
                self._state = "Connected"
            elif self.spa_state == GeckoSpaState.CONNECTING:
                self._state = "Connecting..."
            elif self.spa_state == GeckoSpaState.ERROR_RF_FAULT:
                self._state = "Lost contact with spa (RFERR)"
            elif self.spa_state == GeckoSpaState.ERROR_PING_MISSED:
                self._state = "Lost contact with in.touch2 module"
            elif self.spa_state == GeckoSpaState.ERROR_NEEDS_ATTENTION:
                self._state = "Needs attention, check logs"
            elif self.spa_state == GeckoSpaState.LOCATING_SPAS:
                self._state = "Searching for spas"
            elif self.spa_state == GeckoSpaState.ERROR_SPA_NOT_FOUND:
                self._state = "Cannot find spa, check logs"
            else:
                self._state = f"{self.spa_state}"
            self._on_change()

        def __repr__(self):
            return f"{self.name}: {self.state}"

    def __init__(self, client_uuid: str, **kwargs: str) -> None:
        """Initialize a SpaMan class

        The preferred pattern is to derive a class from SpaMan and then use it in
        an async with

        ```async with MySpaMan(client_uuid, **kwargs) as SpaMan:```

        If you leave **kwargs empty, then you are responsible for co-ordinating calls
        to methods such as async_locate_spas, async_connect_to_spa and async_connect.

        **kwargs can contain the following items

            spa_address:    The IP address of the spa. Useful if the spa is on a sub-net
            spa_identifier: The ID of a spa (as a string)
            spa_name:       The name of the spa. Useful for status feedback when spa
                            cannot be contacted

        If any of the **kwargs are provided then the spam manager will automatically
        run the sequence discover and connect
        """
        AsyncTasks.__init__(self)
        self._client_id = GeckoConstants.FORMAT_CLIENT_IDENTIFIER.format(
            client_uuid
        ).encode(GeckoConstants.MESSAGE_ENCODING)

        # Optional parameters as supplied from config
        self._spa_address: Optional[str] = kwargs.get("spa_address", None)
        if self._spa_address == "":
            self._spa_address = None
        self._spa_identifier: Optional[str] = kwargs.get("spa_identifier", None)
        if self._spa_identifier == "":
            self._spa_identifier = None
        self._spa_name: Optional[str] = kwargs.get("spa_name", None)
        if self._spa_name == "":
            self._spa_name = None

        self._spa_descriptors: Optional[List[GeckoAsyncSpaDescriptor]] = None
        self._facade: Optional[GeckoAsyncFacade] = None
        self._spa: Optional[GeckoAsyncSpa] = None
        self._spa_state = GeckoSpaState.IDLE

        self._status_sensor: Optional[GeckoAsyncSpaMan.StatusSensor] = None
        self._reconnect_button: Optional[GeckoAsyncSpaMan.ReconnectButton] = None
        self._ping_sensor: Optional[GeckoAsyncSpaMan.PingSensor] = None

    ########################################################################
    #
    #   Usage helpers
    #

    async def __aenter__(self) -> GeckoAsyncSpaMan:
        await AsyncTasks.__aenter__(self)
        await self._handle_event(GeckoSpaEvent.SPA_MAN_ENTER)
        self.add_task(self._sequence_pump(), "Sequence Pump", "SPAMAN")
        return self

    async def __aexit__(self, *exc_info) -> None:
        self.cancel_key_tasks("SPAMAN")
        await self._handle_event(GeckoSpaEvent.SPA_MAN_EXIT, exc_info=exc_info)
        await AsyncTasks.__aexit__(self, exc_info)

    ########################################################################
    #
    #   Public methods
    #

    async def async_reset(self) -> None:
        """Reset the spa manager"""
        self._spa_descriptors = None
        if self._facade is not None:
            self._facade = None
        if self._spa is not None:
            await self._spa.disconnect()
            self._spa = None
        self._spa_state = GeckoSpaState.IDLE

    async def async_locate_spas(
        self, spa_address: Optional[str] = None, spa_identifier: Optional[str] = None
    ) -> Optional[List[GeckoAsyncSpaDescriptor]]:
        """Locate spas on this network

        This API will return a list of GeckoAsyncSpaDescriptor that were
        found on the network. If there are none found, then the return will be
        None. Events will be issued as the locating process proceeds


        """
        try:
            await self._handle_event(GeckoSpaEvent.LOCATING_STARTED)
            locator = GeckoAsyncLocator(
                self,
                self._handle_event,
                spa_address=spa_address,
                spa_identifier=spa_identifier,
            )
            await locator.discover()
            self._spa_descriptors = locator.spas
            del locator

        finally:
            await self._handle_event(
                GeckoSpaEvent.LOCATING_FINISHED,
                spa_descriptors=self._spa_descriptors,
            )

        return self._spa_descriptors

    async def async_connect_to_spa(self, spa_descriptor) -> Optional[GeckoAsyncFacade]:
        """Connect to spa.

        This API will connect to the specified spa using the supplied descriptor"""
        assert self._facade is None

        try:
            self._spa_name = spa_descriptor.name
            await self._handle_event(GeckoSpaEvent.CONNECTION_STARTED)
            self._spa = GeckoAsyncSpa(
                self._client_id, spa_descriptor, self, self._handle_event
            )
            await self._spa.connect()
            # Check state now
            if self._spa_state == GeckoSpaState.SPA_READY:
                self._facade = GeckoAsyncFacade(self._spa, self)

        finally:
            await self._handle_event(
                GeckoSpaEvent.CONNECTION_FINISHED, facade=self._facade
            )

        # return facade
        return self._facade

    async def async_connect(
        self, spa_identifier: str, spa_address: Optional[str] = None
    ) -> Optional[GeckoAsyncFacade]:
        """Connect to spa.

        This API will connect to the specified spa by doing a search with the
        supplied information. This is probably the API most commonly used by
        automation systems to avoid storing too much information in configuration"""
        _LOGGER.debug("async_connect: ID:%s ADDR:%s", spa_identifier, spa_address)

        spa_descriptors = await self.async_locate_spas(spa_address, spa_identifier)

        assert spa_descriptors is not None
        if len(spa_descriptors) == 0:
            await self._handle_event(
                GeckoSpaEvent.SPA_NOT_FOUND,
                spa_address=spa_address,
                spa_identifier=spa_identifier,
            )
            return None

        return await self.async_connect_to_spa(spa_descriptors[0])

    async def async_set_spa_info(
        self,
        spa_address: Optional[str],
        spa_identifier: Optional[str],
        spa_name: Optional[str],
    ) -> None:
        """Set the spa information so that the sequence pump can run the locate and
        connect phases"""
        _LOGGER.debug(
            "set_spa_info: ADDR:%s ID:%s NAME:%s", spa_address, spa_identifier, spa_name
        )
        self._spa_address = spa_address
        self._spa_identifier = spa_identifier
        self._spa_name = spa_name
        await self.async_reset()

    async def wait_for_descriptors(self) -> None:
        """Wait for descriptors to be available"""
        while self._spa_descriptors is None:
            await asyncio.sleep(0)

    async def wait_for_facade(self) -> bool:
        """Wait for facade to be available"""
        while self._facade is None:
            await asyncio.sleep(0)
            if self.spa_state == GeckoSpaState.ERROR_SPA_NOT_FOUND:
                return False
        return True

    ########################################################################
    #
    #   Properties
    #

    @property
    def unique_id(self) -> str:
        """A unique id for the spa manager"""
        assert self._spa_identifier is not None
        return f"{self._spa_identifier.replace(':', '')}"

    @property
    def spa_name(self) -> str:
        """The name of the spa being managed"""
        assert self._spa_name is not None
        return self._spa_name

    @property
    def spa_descriptors(self) -> Optional[List[GeckoAsyncSpaDescriptor]]:
        """Get a list of the discovered spas, or None"""
        return self._spa_descriptors

    @property
    def facade(self) -> Optional[GeckoAsyncFacade]:
        """Get the connected facade, or None"""
        return self._facade

    @property
    def spa_state(self) -> GeckoSpaState:
        """Get the spa state"""
        return self._spa_state

    @property
    def status_sensor(self) -> Optional[GeckoAsyncSpaMan.StatusSensor]:
        return self._status_sensor

    @property
    def reconnect_button(self) -> Optional[GeckoAsyncSpaMan.ReconnectButton]:
        return self._reconnect_button

    @property
    def ping_sensor(self) -> Optional[GeckoAsyncSpaMan.PingSensor]:
        return self._ping_sensor

    def __str__(self) -> str:
        if self.status_sensor is not None:
            return f"{self.status_sensor}"
        return "Initializing"

    ########################################################################
    #
    #   Abstract methods
    #
    @abstractmethod
    async def handle_event(self, event: GeckoSpaEvent, **kwargs) -> None:
        pass

    ########################################################################
    #
    #   Private methods
    #

    async def _handle_event(self, event: GeckoSpaEvent, **kwargs) -> None:

        if (
            self._status_sensor is None
            and self._spa_identifier is not None
            and self._spa_name is not None
        ):
            self._status_sensor = GeckoAsyncSpaMan.StatusSensor(self)
            await self._handle_event(GeckoSpaEvent.CLIENT_HAS_STATUS_SENSOR)

        # Do any pre-processing of the event, such as setting the state or
        # updating the status line
        if event == GeckoSpaEvent.LOCATING_STARTED:
            self._spa_state = GeckoSpaState.LOCATING_SPAS

        elif event == GeckoSpaEvent.LOCATING_FINISHED:
            self._spa_state = GeckoSpaState.IDLE

        elif event == GeckoSpaEvent.SPA_NOT_FOUND:
            self._spa_state = GeckoSpaState.ERROR_SPA_NOT_FOUND

        elif event == GeckoSpaEvent.CONNECTION_STARTED:
            self._spa_state = GeckoSpaState.CONNECTING
            self._reconnect_button = GeckoAsyncSpaMan.ReconnectButton(self)
            await self._handle_event(GeckoSpaEvent.CLIENT_HAS_RECONNECT_BUTTON)

        elif event == GeckoSpaEvent.CONNECTION_GOT_CHANNEL:
            self._ping_sensor = GeckoAsyncSpaMan.PingSensor(self)
            await self._handle_event(GeckoSpaEvent.CLIENT_HAS_PING_SENSOR)

        elif event == GeckoSpaEvent.CONNECTION_SPA_COMPLETE:
            self._spa_state = GeckoSpaState.SPA_READY

        elif event == GeckoSpaEvent.CONNECTION_FINISHED:
            if self._facade is not None:
                self._spa_state = GeckoSpaState.CONNECTED
                await self._handle_event(GeckoSpaEvent.CLIENT_FACADE_IS_READY)

        elif event == GeckoSpaEvent.RUNNING_PING_NO_RESPONSE:
            if self._spa_state == GeckoSpaState.CONNECTED:
                self._spa_state = GeckoSpaState.ERROR_PING_MISSED
                await self._handle_event(GeckoSpaEvent.CLIENT_FACADE_TEARDOWN)

        elif event == GeckoSpaEvent.RUNNING_PING_RECEIVED:
            if self._spa_state in (
                GeckoSpaState.ERROR_PING_MISSED,
                GeckoSpaState.ERROR_RF_FAULT,
                GeckoSpaState.ERROR_NEEDS_ATTENTION,
            ):
                await self.async_reset()

        elif event == GeckoSpaEvent.ERROR_RF_ERROR:
            if self._spa_state == GeckoSpaState.CONNECTED:
                self._spa_state = GeckoSpaState.ERROR_RF_FAULT
                await self._handle_event(GeckoSpaEvent.CLIENT_FACADE_TEARDOWN)

        elif event == GeckoSpaEvent.RUNNING_SPA_DISCONNECTED:
            if self._spa_state == GeckoSpaState.CONNECTED:
                self._spa_state = GeckoSpaState.IDLE
                await self._handle_event(GeckoSpaEvent.CLIENT_FACADE_TEARDOWN)

        elif event in (
            GeckoSpaEvent.CONNECTION_PROTOCOL_RETRY_COUNT_EXCEEDED,
            GeckoSpaEvent.ERROR_PROTOCOL_RETRY_COUNT_EXCEEDED,
            GeckoSpaEvent.ERROR_TOO_MANY_RF_ERRORS,
        ):
            self._spa_state = GeckoSpaState.ERROR_NEEDS_ATTENTION

        elif event == GeckoSpaEvent.RUNNING_SPA_WATER_CARE_ERROR:
            assert self.facade is not None
            assert self._spa is not None
            self.facade._water_care.change_watercare_mode(
                await self._spa.async_get_watercare()
            )

        if self._status_sensor is not None:
            self._status_sensor.on_event(event)

        # Call the abstract method to allow derived classes to do useful work
        # such as disconnecting handlers when the spa needs to reconnect
        # after protocol failure
        await self.handle_event(event, **kwargs)

        # Any post-processing goes here

    async def _sequence_pump(self) -> None:
        """SpaMan sequence pump coordinates running the manager from the
        parameterized constructor and machine state"""
        _LOGGER.debug("SpaMan sequence pump started")

        try:
            while True:

                if self.spa_state == GeckoSpaState.IDLE:

                    if self._spa_identifier is not None:

                        if self._facade is None:
                            await self.async_connect(
                                self._spa_identifier, self._spa_address
                            )

                    elif self._spa_descriptors is None:
                        await self.async_locate_spas(self._spa_address)

                await asyncio.sleep(0)

        except asyncio.CancelledError:
            _LOGGER.debug("Spaman sequence pump cancelled")
            raise



#!/usr/bin/python3
"""
    GeckoLogStruct - A class to manage the LogStruct for 'InXM v2'
"""

from . import (
    GeckoByteStructAccessor,
    GeckoWordStructAccessor,
    GeckoTimeStructAccessor,
    GeckoBoolStructAccessor,
    GeckoEnumStructAccessor,
    GeckoTempStructAccessor,
)

# Constants for this class
ACCPQI = 313
AHEOCT = 11
AKQXPI = 296
AMJMAO = "".join(chr(c) for c in [82, 72, 95, 82, 101, 103, 83, 108, 111, 112, 101])
AOAWBS = "".join(
    chr(c) for c in [82, 72, 95, 78, 111, 70, 108, 111, 82, 101, 116, 114, 121]
)
AONPYY = 307
ASSAKQ = "".join(
    chr(c)
    for c in [
        76,
        101,
        97,
        114,
        110,
        105,
        110,
        103,
        67,
        117,
        114,
        114,
        101,
        110,
        116,
        79,
        85,
        84,
        54,
        65,
    ]
)
AWBSIR = "".join(
    chr(c) for c in [82, 72, 95, 78, 111, 70, 108, 111, 88, 84, 114, 105, 101, 115]
)
BFEGZU = "".join(chr(c) for c in [80, 50])
BIAMJM = 257
BLKXSJ = "".join(
    chr(c) for c in [83, 87, 77, 80, 117, 114, 103, 101, 95, 83, 117, 115, 112]
)
BMJVHF = "".join(chr(c) for c in [66, 114, 77, 101, 110, 117])
BQFYLJ = "".join(chr(c) for c in [82, 72, 95, 83, 87, 84, 114, 105, 97, 99, 79, 72])
BSIRYX = "".join(chr(c) for c in [82, 72, 95, 72, 87, 95, 72, 76])
BSKSOK = "".join(chr(c) for c in [79, 78])
BWJYKL = "".join(chr(c) for c in [70, 117, 115, 101, 51, 69, 114, 114])
CCPQIP = "".join(chr(c) for c in [70, 105, 108, 116, 80, 117, 114, 103, 101])
CPQIPO = 269
CQBMJV = 256
CTHBSK = "".join(chr(c) for c in [85, 68, 95, 80, 53])
CVYYPI = "".join(
    chr(c)
    for c in [
        76,
        101,
        97,
        114,
        110,
        105,
        110,
        103,
        67,
        117,
        114,
        114,
        101,
        110,
        116,
        79,
        85,
        84,
        50,
        65,
    ]
)
CWAONP = 303
CXQIEF = "".join(chr(c) for c in [76, 79, 87])
ECVYYP = 282
EFJTAC = 310
EFXQGL = 2
EGZUQE = "".join(chr(c) for c in [80, 52])
EJNIBX = 256
EKCWAO = 306
EOCTHB = "".join(chr(c) for c in [85, 68, 95, 80, 52])
EXLSXU = "".join(chr(c) for c in [79, 90, 79, 78, 69])
FEFJTA = "".join(
    chr(c) for c in [82, 101, 109, 111, 116, 101, 70, 105, 108, 116, 68, 117, 114]
)
FEGZUQ = "".join(chr(c) for c in [80, 51])
FJBIAM = 263
FJTACC = "".join(
    chr(c)
    for c in [
        82,
        101,
        109,
        111,
        116,
        101,
        79,
        110,
        122,
        101,
        110,
        65,
        99,
        116,
        105,
        111,
        110,
    ]
)
FTHECV = "".join(
    chr(c)
    for c in [
        76,
        101,
        97,
        114,
        110,
        105,
        110,
        103,
        67,
        117,
        114,
        114,
        101,
        110,
        116,
        79,
        85,
        84,
        49,
        65,
    ]
)
FTSIFJ = "".join(chr(c) for c in [79, 72, 67, 111, 110, 100, 105, 116, 105, 111, 110])
FWRKIN = "".join(chr(c) for c in [82, 72, 95, 70, 108, 111, 119, 67, 104, 101, 99, 107])
FXQGLR = 4
FYLJUI = "".join(chr(c) for c in [82, 72, 95, 72, 114, 75, 105, 110])
GLRAHE = "".join(chr(c) for c in [71, 101, 99, 107, 111])
GQPLSP = "".join(
    chr(c)
    for c in [82, 101, 103, 79, 118, 101, 114, 72, 101, 97, 116, 70, 108, 97, 103]
)
GYOUSP = 301
GZUQEX = "".join(chr(c) for c in [80, 53])
HBSKSO = "".join(chr(c) for c in [85, 68, 95, 66, 76, 79, 87, 69, 82])
HECVYY = "".join(
    chr(c)
    for c in [
        76,
        101,
        97,
        114,
        110,
        105,
        110,
        103,
        67,
        117,
        114,
        114,
        101,
        110,
        116,
        79,
        85,
        84,
        49,
        66,
    ]
)
HEOCTH = "".join(chr(c) for c in [82, 68])
HFTHEC = 300
HIUSOO = "".join(chr(c) for c in [32])
HUOJRJ = "".join(chr(c) for c in [70, 73, 88, 67, 79, 76, 79, 82])
IAMJMA = "".join(
    chr(c)
    for c in [82, 72, 95, 78, 111, 116, 69, 110, 111, 117, 103, 104, 72, 101, 97, 116]
)
ICXQIE = "".join(chr(c) for c in [79, 70, 70])
IEFXQG = 14
IFJBIA = "".join(chr(c) for c in [82, 72, 95, 84, 114, 105, 97, 99, 84, 101, 109, 112])
IGYOUS = "".join(
    chr(c) for c in [69, 120, 116, 80, 114, 111, 98, 101, 65, 99, 116, 105, 118, 101]
)
IKFWRK = 260
INEJNI = "".join(chr(c) for c in [76, 73])
IPIVLA = 288
IPOUYN = "".join(
    chr(c)
    for c in [
        70,
        105,
        108,
        116,
        80,
        117,
        114,
        103,
        101,
        83,
        117,
        115,
        112,
        101,
        110,
        100,
        85,
        68,
    ]
)
IRYXBQ = "".join(
    chr(c) for c in [82, 72, 95, 82, 101, 103, 79, 118, 101, 114, 72, 101, 97, 116]
)
IUSOOQ = "".join(chr(c) for c in [81, 85, 73, 69, 84])
IUXFEF = "".join(chr(c) for c in [78, 69, 87])
IVLASS = 290
JBIAMJ = "".join(chr(c) for c in [82, 72, 95, 67, 111, 109, 109, 69, 114, 114])
JHIUSO = "".join(
    chr(c) for c in [85, 68, 95, 81, 85, 73, 69, 84, 95, 80, 65, 85, 83, 69]
)
JIGYOU = "".join(
    chr(c)
    for c in [
        80,
        114,
        111,
        103,
        69,
        99,
        111,
        110,
        111,
        109,
        121,
        65,
        99,
        116,
        105,
        118,
        101,
    ]
)
JMAOAW = "".join(
    chr(c) for c in [82, 72, 95, 68, 117, 116, 121, 67, 121, 99, 108, 101, 69, 114, 114]
)
JMCBFE = "".join(chr(c) for c in [80, 49])
JNIBXY = 511
JRJHIU = "".join(chr(c) for c in [85, 68, 95, 76, 73, 71, 72, 84, 49, 50, 48])
JTACCP = 312
JUIKFW = "".join(chr(c) for c in [82, 72, 95, 72, 87, 75, 105, 110, 101, 116, 105, 99])
JUTYEK = 274
JVHFTH = 6
JWMNZM = "".join(chr(c) for c in [72, 101, 97, 116, 105, 110, 103])
JYKLGQ = "".join(chr(c) for c in [70, 117, 115, 101, 49, 69, 114, 114])
JYMOUN = 268
KCWAON = "".join(
    chr(c)
    for c in [
        86,
        83,
        80,
        51,
        95,
        65,
        118,
        97,
        110,
        116,
        80,
        108,
        97,
        110,
        83,
        112,
        101,
        101,
        100,
    ]
)
KFWRKI = "".join(
    chr(c)
    for c in [82, 72, 95, 70, 108, 111, 119, 68, 101, 116, 101, 99, 116, 101, 100]
)
KLGQPL = "".join(chr(c) for c in [83, 117, 112, 112, 108, 121, 69, 114, 114])
KPHUOJ = "".join(chr(c) for c in [85, 68, 95, 65, 85, 88, 50])
KQXPIC = "".join(
    chr(c) for c in [67, 117, 114, 114, 101, 110, 116, 72, 101, 97, 116, 101, 114]
)
KSOKPH = 8
KXSJWM = 265
LASSAK = 292
LGQPLS = "".join(chr(c) for c in [83, 99, 97, 110, 69, 114, 114])
LIUXFE = "".join(chr(c) for c in [83, 84, 65, 82, 84])
LJUIKF = "".join(chr(c) for c in [82, 72, 95, 83, 119, 75, 105, 110, 84, 101, 109, 112])
LKXSJW = "".join(
    chr(c)
    for c in [68, 105, 115, 112, 108, 97, 121, 101, 100, 84, 101, 109, 112, 95, 71]
)
LRAHEO = "".join(chr(c) for c in [85, 68, 95, 80, 51])
LSPFTS = "".join(chr(c) for c in [82, 101, 108, 97, 121, 83, 116, 117, 99, 107])
LSXUJU = "".join(chr(c) for c in [76, 49, 50, 48])
MAOAWB = 258
MCBFEG = 316
MJIGYO = 271
MJMAOA = "".join(chr(c) for c in [82, 72, 95, 76, 111, 119, 70, 108, 111])
MJVHFT = 279
MNZMJI = "".join(chr(c) for c in [67, 111, 111, 108, 105, 110, 103, 68, 111, 119, 110])
MOUNBL = "".join(chr(c) for c in [77, 69, 68, 73, 85, 77])
NBLKXS = "".join(chr(c) for c in [83, 87, 77, 80, 117, 114, 103, 101])
NPYYLI = 309
NQJYMO = 267
NRSJMC = 305
NZMJIG = "".join(
    chr(c)
    for c in [
        75,
        105,
        110,
        101,
        116,
        105,
        99,
        80,
        117,
        114,
        103,
        101,
        65,
        99,
        116,
        105,
        118,
        101,
    ]
)
OAWBSI = "".join(chr(c) for c in [82, 72, 95, 78, 111, 70, 108, 111, 84, 101, 109, 112])
OCTHBS = 10
OKPHUO = 5
ONPYYL = "".join(
    chr(c)
    for c in [
        82,
        101,
        109,
        111,
        116,
        101,
        70,
        105,
        108,
        116,
        65,
        99,
        116,
        105,
        111,
        110,
    ]
)
OOQNRS = "".join(chr(c) for c in [86, 83, 80, 49, 95, 85, 68, 83, 112, 101, 101, 100])
OQNRSJ = 304
OUNBLK = "".join(chr(c) for c in [69, 88, 84, 82, 69, 77, 69])
OUSPBW = 277
OUYNQJ = "".join(chr(c) for c in [70, 105, 108, 116, 101, 114, 79, 84])
PBWJYK = "".join(
    chr(c)
    for c in [
        76,
        101,
        97,
        114,
        110,
        68,
        101,
        116,
        101,
        99,
        116,
        105,
        111,
        110,
        69,
        114,
        114,
    ]
)
PFTSIF = "".join(chr(c) for c in [75, 105, 110, 80, 117, 109, 112, 79, 102, 102])
PHUOJR = "".join(chr(c) for c in [85, 68, 95, 70, 66, 95, 76, 84])
PICXQI = 275
PIPIVL = "".join(
    chr(c)
    for c in [
        76,
        101,
        97,
        114,
        110,
        105,
        110,
        103,
        67,
        117,
        114,
        114,
        101,
        110,
        116,
        79,
        85,
        84,
        51,
        65,
    ]
)
PIVLAS = "".join(
    chr(c)
    for c in [
        76,
        101,
        97,
        114,
        110,
        105,
        110,
        103,
        67,
        117,
        114,
        114,
        101,
        110,
        116,
        79,
        85,
        84,
        52,
        65,
    ]
)
PLSPFT = "".join(
    chr(c) for c in [65, 109, 98, 105, 97, 110, 116, 79, 72, 76, 101, 118, 101, 108, 49]
)
POUYNQ = "".join(
    chr(c)
    for c in [
        70,
        105,
        108,
        116,
        67,
        108,
        101,
        97,
        110,
        83,
        117,
        115,
        112,
        101,
        110,
        100,
        85,
        68,
    ]
)
PQIPOU = "".join(chr(c) for c in [70, 105, 108, 116, 67, 108, 101, 97, 110])
PYYLIU = "".join(chr(c) for c in [85, 83, 69, 95, 73, 78, 84, 69, 82, 78, 65, 76])
QBMJVH = "".join(chr(c) for c in [65, 76, 76])
QEXLSX = "".join(chr(c) for c in [67, 80])
QFYLJU = "".join(chr(c) for c in [82, 72, 95, 72, 119, 84, 114, 105, 97, 99, 79, 72])
QGLRAH = 12
QIPOUY = "".join(
    chr(c)
    for c in [
        70,
        105,
        108,
        116,
        67,
        108,
        101,
        97,
        110,
        83,
        117,
        115,
        112,
        101,
        110,
        100,
        101,
        100,
    ]
)
QJYMOU = "".join(chr(c) for c in [83, 87, 77, 95, 82, 105, 115, 107])
QNRSJM = "".join(chr(c) for c in [86, 83, 80, 51, 95, 85, 68, 83, 112, 101, 101, 100])
QPLSPF = "".join(
    chr(c) for c in [65, 109, 98, 105, 97, 110, 116, 79, 72, 76, 101, 118, 101, 108, 50]
)
QXPICX = 298
RJHIUS = 1
RKINEJ = "".join(
    chr(c)
    for c in [82, 72, 95, 70, 108, 111, 67, 104, 101, 99, 107, 82, 101, 97, 100, 121]
)
RSJMCB = "".join(
    chr(c) for c in [76, 105, 103, 104, 116, 73, 110, 116, 101, 110, 115, 105, 116, 121]
)
RYXBQF = "".join(chr(c) for c in [82, 72, 95, 72, 114, 84, 114, 105, 97, 99, 80, 114])
SAKQXP = "".join(
    chr(c)
    for c in [
        76,
        101,
        97,
        114,
        110,
        105,
        110,
        103,
        67,
        117,
        114,
        114,
        101,
        110,
        116,
        79,
        85,
        84,
        55,
        65,
    ]
)
SIFJBI = 261
SIRYXB = "".join(
    chr(c) for c in [82, 72, 95, 82, 101, 103, 80, 114, 111, 98, 101, 69, 114, 114]
)
SJMCBF = 308
SJWMNZ = "".join(chr(c) for c in [67, 104, 101, 99, 107, 70, 108, 111, 119])
SOKPHU = "".join(chr(c) for c in [85, 68, 95, 65, 85, 88, 49])
SOOQNR = 0
SPBWJY = "".join(
    chr(c)
    for c in [
        78,
        111,
        72,
        101,
        97,
        116,
        101,
        114,
        67,
        117,
        114,
        114,
        101,
        110,
        116,
        69,
        114,
        114,
    ]
)
SPFTSI = "".join(chr(c) for c in [114, 72, 73, 100])
SSAKQX = 294
SXUJUT = "".join(chr(c) for c in [70, 65, 78])
TACCPQ = "".join(
    chr(c) for c in [82, 101, 109, 111, 116, 101, 79, 110, 122, 101, 110, 68, 117, 114]
)
THBSKS = 9
THECVY = 280
TSIFJB = "".join(chr(c) for c in [82, 72, 95, 87, 97, 116, 101, 114, 84, 101, 109, 112])
TYEKCW = 302
UIKFWR = "".join(
    chr(c)
    for c in [82, 72, 95, 72, 101, 97, 116, 101, 114, 68, 105, 115, 97, 98, 108, 101]
)
UJUTYE = "".join(chr(c) for c in [82, 72, 95, 68, 117, 116, 121, 67, 121, 99, 108, 101])
UOJRJH = "".join(chr(c) for c in [83, 67, 65, 78])
UQEXLS = "".join(chr(c) for c in [66, 76, 79, 87, 69, 82])
USPBWJ = "".join(
    chr(c)
    for c in [
        67,
        111,
        110,
        116,
        97,
        99,
        116,
        111,
        114,
        67,
        111,
        105,
        108,
        70,
        97,
        105,
        108,
        69,
        114,
        114,
    ]
)
UTYEKC = "".join(
    chr(c)
    for c in [
        86,
        83,
        80,
        49,
        95,
        65,
        118,
        97,
        110,
        116,
        80,
        108,
        97,
        110,
        83,
        112,
        101,
        101,
        100,
    ]
)
UXFEFJ = "".join(chr(c) for c in [70, 73, 76, 84, 69, 82])
UYNQJY = "".join(chr(c) for c in [67, 80, 79, 84])
VHFTHE = "".join(chr(c) for c in [72, 101, 97, 116, 101, 114, 80, 104, 97, 115, 101])
VLASSA = "".join(
    chr(c)
    for c in [
        76,
        101,
        97,
        114,
        110,
        105,
        110,
        103,
        67,
        117,
        114,
        114,
        101,
        110,
        116,
        79,
        85,
        84,
        53,
        65,
    ]
)
VYYPIP = 284
WAONPY = "".join(
    chr(c)
    for c in [
        86,
        83,
        80,
        51,
        95,
        77,
        105,
        110,
        105,
        109,
        117,
        109,
        83,
        112,
        101,
        101,
        100,
    ]
)
WBSIRY = "".join(chr(c) for c in [82, 72, 95, 72, 82, 95, 72, 76])
WJYKLG = "".join(chr(c) for c in [70, 117, 115, 101, 50, 69, 114, 114])
WMNZMJ = 270
WRKINE = "".join(
    chr(c)
    for c in [
        82,
        72,
        95,
        72,
        101,
        97,
        116,
        101,
        114,
        83,
        117,
        115,
        112,
        101,
        110,
        100,
        101,
        100,
    ]
)
XBQFYL = "".join(chr(c) for c in [82, 72, 95, 72, 114, 84, 114, 105, 97, 99, 79, 72])
XLSXUJ = 3
XPICXQ = "".join(chr(c) for c in [85, 68, 95, 80, 49])
XQGLRA = "".join(chr(c) for c in [85, 68, 95, 80, 50])
XQIEFX = "".join(chr(c) for c in [72, 73, 71, 72])
XSJWMN = "".join(chr(c) for c in [80, 117, 109, 112, 82, 117, 110])
XUJUTY = 7
YEKCWA = "".join(
    chr(c)
    for c in [
        86,
        83,
        80,
        49,
        95,
        77,
        105,
        110,
        105,
        109,
        117,
        109,
        83,
        112,
        101,
        101,
        100,
    ]
)
YKLGQP = 278
YLIUXF = "".join(chr(c) for c in [83, 84, 79, 80])
YLJUIK = "".join(
    chr(c) for c in [82, 72, 95, 72, 114, 75, 105, 110, 78, 111, 70, 108, 111]
)
YMOUNB = "".join(chr(c) for c in [78, 79])
YNQJYM = "".join(chr(c) for c in [83, 87, 77, 95, 65, 99, 116, 105, 118, 101])
YOUSPB = "".join(
    chr(c)
    for c in [84, 104, 101, 114, 109, 105, 115, 116, 97, 110, 99, 101, 69, 114, 114]
)
YPIPIV = 286
YXBQFY = 259
YYLIUX = "".join(chr(c) for c in [73, 68, 76, 69])
YYPIPI = "".join(
    chr(c)
    for c in [
        76,
        101,
        97,
        114,
        110,
        105,
        110,
        103,
        67,
        117,
        114,
        114,
        101,
        110,
        116,
        79,
        85,
        84,
        50,
        66,
    ]
)
ZCQBMJ = "".join(chr(c) for c in [72, 111, 117, 114, 115])
ZMJIGY = "".join(
    chr(c) for c in [69, 99, 111, 110, 111, 109, 121, 65, 99, 116, 105, 118, 101]
)
ZUQEXL = 315
CBFEGZ = [ICXQIE, XQIEFX, CXQIEF]
KINEJN = []
NEJNIB = [
    JMCBFE,
    BFEGZU,
    FEGZUQ,
    EGZUQE,
    GZUQEX,
    UQEXLS,
    QEXLSX,
    EXLSXU,
    LSXUJU,
    SXUJUT,
    UJUTYE,
    UTYEKC,
    YEKCWA,
    KCWAON,
    WAONPY,
    INEJNI,
]
OJRJHI = [ICXQIE, HUOJRJ, UOJRJH]
QIEFXQ = [ICXQIE, CXQIEF, XQIEFX]
RAHEOC = [ICXQIE, XQIEFX]
SKSOKP = [ICXQIE, BSKSOK]
UNBLKX = [YMOUNB, CXQIEF, MOUNBL, XQIEFX, OUNBLK]
USOOQN = [HIUSOO, IUSOOQ]
XFEFJT = [PYYLIU, YYLIUX, YLIUXF, LIUXFE, IUXFEF, UXFEFJ]


class GeckoLogStruct:
    def __init__(self, struct_):
        self.struct = struct_

    @property
    def version(self):
        return EFXQGL

    @property
    def begin(self):
        return EJNIBX

    @property
    def end(self):
        return JNIBXY

    @property
    def all_device_keys(self):
        return NEJNIB

    @property
    def user_demand_keys(self):
        return KINEJN

    @property
    def accessors(self):
        return {
            ZCQBMJ: GeckoByteStructAccessor(self.struct, ZCQBMJ, CQBMJV, QBMJVH),
            BMJVHF: GeckoBoolStructAccessor(self.struct, BMJVHF, MJVHFT, JVHFTH, None),
            VHFTHE: GeckoByteStructAccessor(self.struct, VHFTHE, HFTHEC, None),
            FTHECV: GeckoWordStructAccessor(self.struct, FTHECV, THECVY, None),
            HECVYY: GeckoWordStructAccessor(self.struct, HECVYY, ECVYYP, None),
            CVYYPI: GeckoWordStructAccessor(self.struct, CVYYPI, VYYPIP, None),
            YYPIPI: GeckoWordStructAccessor(self.struct, YYPIPI, YPIPIV, None),
            PIPIVL: GeckoWordStructAccessor(self.struct, PIPIVL, IPIVLA, None),
            PIVLAS: GeckoWordStructAccessor(self.struct, PIVLAS, IVLASS, None),
            VLASSA: GeckoWordStructAccessor(self.struct, VLASSA, LASSAK, None),
            ASSAKQ: GeckoWordStructAccessor(self.struct, ASSAKQ, SSAKQX, None),
            SAKQXP: GeckoWordStructAccessor(self.struct, SAKQXP, AKQXPI, None),
            KQXPIC: GeckoWordStructAccessor(self.struct, KQXPIC, QXPICX, QBMJVH),
            XPICXQ: GeckoEnumStructAccessor(
                self.struct, XPICXQ, PICXQI, IEFXQG, QIEFXQ, EFXQGL, FXQGLR, QBMJVH
            ),
            XQGLRA: GeckoEnumStructAccessor(
                self.struct, XQGLRA, PICXQI, QGLRAH, QIEFXQ, EFXQGL, FXQGLR, GLRAHE
            ),
            LRAHEO: GeckoEnumStructAccessor(
                self.struct, LRAHEO, PICXQI, AHEOCT, RAHEOC, EFXQGL, EFXQGL, HEOCTH
            ),
            EOCTHB: GeckoEnumStructAccessor(
                self.struct, EOCTHB, PICXQI, OCTHBS, RAHEOC, EFXQGL, EFXQGL, QBMJVH
            ),
            CTHBSK: GeckoEnumStructAccessor(
                self.struct, CTHBSK, PICXQI, THBSKS, RAHEOC, EFXQGL, EFXQGL, QBMJVH
            ),
            HBSKSO: GeckoEnumStructAccessor(
                self.struct, HBSKSO, PICXQI, KSOKPH, SKSOKP, EFXQGL, EFXQGL, QBMJVH
            ),
            SOKPHU: GeckoEnumStructAccessor(
                self.struct, SOKPHU, PICXQI, OKPHUO, SKSOKP, EFXQGL, EFXQGL, QBMJVH
            ),
            KPHUOJ: GeckoEnumStructAccessor(
                self.struct, KPHUOJ, PICXQI, FXQGLR, SKSOKP, EFXQGL, EFXQGL, QBMJVH
            ),
            PHUOJR: GeckoEnumStructAccessor(
                self.struct, PHUOJR, PICXQI, EFXQGL, OJRJHI, EFXQGL, FXQGLR, QBMJVH
            ),
            JRJHIU: GeckoEnumStructAccessor(
                self.struct, JRJHIU, PICXQI, RJHIUS, SKSOKP, EFXQGL, EFXQGL, QBMJVH
            ),
            JHIUSO: GeckoEnumStructAccessor(
                self.struct, JHIUSO, PICXQI, SOOQNR, USOOQN, EFXQGL, EFXQGL, QBMJVH
            ),
            OOQNRS: GeckoByteStructAccessor(self.struct, OOQNRS, OQNRSJ, QBMJVH),
            QNRSJM: GeckoByteStructAccessor(self.struct, QNRSJM, NRSJMC, QBMJVH),
            RSJMCB: GeckoByteStructAccessor(self.struct, RSJMCB, SJMCBF, QBMJVH),
            JMCBFE: GeckoEnumStructAccessor(
                self.struct, JMCBFE, MCBFEG, SOOQNR, CBFEGZ, None, FXQGLR, None
            ),
            BFEGZU: GeckoEnumStructAccessor(
                self.struct, BFEGZU, MCBFEG, EFXQGL, CBFEGZ, None, FXQGLR, None
            ),
            FEGZUQ: GeckoEnumStructAccessor(
                self.struct, FEGZUQ, MCBFEG, FXQGLR, CBFEGZ, None, FXQGLR, None
            ),
            EGZUQE: GeckoEnumStructAccessor(
                self.struct, EGZUQE, MCBFEG, JVHFTH, CBFEGZ, None, FXQGLR, None
            ),
            GZUQEX: GeckoEnumStructAccessor(
                self.struct, GZUQEX, ZUQEXL, SOOQNR, RAHEOC, None, EFXQGL, None
            ),
            UQEXLS: GeckoEnumStructAccessor(
                self.struct, UQEXLS, ZUQEXL, RJHIUS, RAHEOC, None, EFXQGL, None
            ),
            QEXLSX: GeckoEnumStructAccessor(
                self.struct, QEXLSX, ZUQEXL, EFXQGL, SKSOKP, None, EFXQGL, None
            ),
            EXLSXU: GeckoEnumStructAccessor(
                self.struct, EXLSXU, ZUQEXL, XLSXUJ, SKSOKP, None, EFXQGL, None
            ),
            LSXUJU: GeckoEnumStructAccessor(
                self.struct, LSXUJU, ZUQEXL, FXQGLR, SKSOKP, None, EFXQGL, None
            ),
            SXUJUT: GeckoEnumStructAccessor(
                self.struct, SXUJUT, ZUQEXL, XUJUTY, SKSOKP, None, EFXQGL, None
            ),
            UJUTYE: GeckoByteStructAccessor(self.struct, UJUTYE, JUTYEK, None),
            UTYEKC: GeckoByteStructAccessor(self.struct, UTYEKC, TYEKCW, None),
            YEKCWA: GeckoByteStructAccessor(self.struct, YEKCWA, EKCWAO, None),
            KCWAON: GeckoByteStructAccessor(self.struct, KCWAON, CWAONP, None),
            WAONPY: GeckoByteStructAccessor(self.struct, WAONPY, AONPYY, None),
            ONPYYL: GeckoEnumStructAccessor(
                self.struct, ONPYYL, NPYYLI, None, XFEFJT, None, JVHFTH, QBMJVH
            ),
            FEFJTA: GeckoTimeStructAccessor(self.struct, FEFJTA, EFJTAC, QBMJVH),
            FJTACC: GeckoEnumStructAccessor(
                self.struct, FJTACC, JTACCP, None, XFEFJT, None, JVHFTH, QBMJVH
            ),
            TACCPQ: GeckoTimeStructAccessor(self.struct, TACCPQ, ACCPQI, QBMJVH),
            CCPQIP: GeckoBoolStructAccessor(self.struct, CCPQIP, CPQIPO, XUJUTY, None),
            PQIPOU: GeckoBoolStructAccessor(self.struct, PQIPOU, CPQIPO, OKPHUO, None),
            QIPOUY: GeckoBoolStructAccessor(self.struct, QIPOUY, CPQIPO, FXQGLR, None),
            IPOUYN: GeckoBoolStructAccessor(self.struct, IPOUYN, MJVHFT, EFXQGL, None),
            POUYNQ: GeckoBoolStructAccessor(self.struct, POUYNQ, MJVHFT, XLSXUJ, None),
            OUYNQJ: GeckoBoolStructAccessor(self.struct, OUYNQJ, MJVHFT, SOOQNR, None),
            UYNQJY: GeckoBoolStructAccessor(self.struct, UYNQJY, MJVHFT, RJHIUS, None),
            YNQJYM: GeckoByteStructAccessor(self.struct, YNQJYM, NQJYMO, None),
            QJYMOU: GeckoEnumStructAccessor(
                self.struct, QJYMOU, JYMOUN, None, UNBLKX, None, KSOKPH, None
            ),
            NBLKXS: GeckoBoolStructAccessor(self.struct, NBLKXS, CPQIPO, XLSXUJ, None),
            BLKXSJ: GeckoBoolStructAccessor(self.struct, BLKXSJ, CPQIPO, EFXQGL, None),
            LKXSJW: GeckoTempStructAccessor(self.struct, LKXSJW, KXSJWM, None),
            XSJWMN: GeckoBoolStructAccessor(self.struct, XSJWMN, CPQIPO, RJHIUS, None),
            SJWMNZ: GeckoBoolStructAccessor(self.struct, SJWMNZ, CPQIPO, SOOQNR, None),
            JWMNZM: GeckoBoolStructAccessor(self.struct, JWMNZM, WMNZMJ, XUJUTY, None),
            MNZMJI: GeckoBoolStructAccessor(self.struct, MNZMJI, WMNZMJ, JVHFTH, None),
            NZMJIG: GeckoBoolStructAccessor(self.struct, NZMJIG, WMNZMJ, OKPHUO, None),
            ZMJIGY: GeckoBoolStructAccessor(self.struct, ZMJIGY, MJIGYO, SOOQNR, None),
            JIGYOU: GeckoBoolStructAccessor(self.struct, JIGYOU, MJIGYO, EFXQGL, None),
            IGYOUS: GeckoBoolStructAccessor(self.struct, IGYOUS, GYOUSP, SOOQNR, None),
            YOUSPB: GeckoBoolStructAccessor(self.struct, YOUSPB, OUSPBW, OKPHUO, None),
            USPBWJ: GeckoBoolStructAccessor(self.struct, USPBWJ, OUSPBW, FXQGLR, None),
            SPBWJY: GeckoBoolStructAccessor(self.struct, SPBWJY, OUSPBW, XLSXUJ, None),
            PBWJYK: GeckoBoolStructAccessor(self.struct, PBWJYK, OUSPBW, EFXQGL, None),
            BWJYKL: GeckoBoolStructAccessor(self.struct, BWJYKL, OUSPBW, RJHIUS, None),
            WJYKLG: GeckoBoolStructAccessor(self.struct, WJYKLG, OUSPBW, SOOQNR, None),
            JYKLGQ: GeckoBoolStructAccessor(self.struct, JYKLGQ, YKLGQP, XUJUTY, None),
            KLGQPL: GeckoBoolStructAccessor(self.struct, KLGQPL, YKLGQP, JVHFTH, None),
            LGQPLS: GeckoBoolStructAccessor(self.struct, LGQPLS, YKLGQP, OKPHUO, None),
            GQPLSP: GeckoBoolStructAccessor(self.struct, GQPLSP, YKLGQP, FXQGLR, None),
            QPLSPF: GeckoBoolStructAccessor(self.struct, QPLSPF, YKLGQP, XLSXUJ, None),
            PLSPFT: GeckoBoolStructAccessor(self.struct, PLSPFT, YKLGQP, EFXQGL, None),
            LSPFTS: GeckoBoolStructAccessor(self.struct, LSPFTS, YKLGQP, RJHIUS, None),
            SPFTSI: GeckoBoolStructAccessor(self.struct, SPFTSI, YKLGQP, SOOQNR, None),
            PFTSIF: GeckoBoolStructAccessor(self.struct, PFTSIF, MJVHFT, FXQGLR, None),
            FTSIFJ: GeckoBoolStructAccessor(self.struct, FTSIFJ, MJVHFT, OKPHUO, None),
            TSIFJB: GeckoTempStructAccessor(self.struct, TSIFJB, SIFJBI, None),
            IFJBIA: GeckoTempStructAccessor(self.struct, IFJBIA, FJBIAM, None),
            JBIAMJ: GeckoBoolStructAccessor(self.struct, JBIAMJ, BIAMJM, XUJUTY, None),
            IAMJMA: GeckoBoolStructAccessor(self.struct, IAMJMA, BIAMJM, XLSXUJ, None),
            AMJMAO: GeckoBoolStructAccessor(self.struct, AMJMAO, BIAMJM, EFXQGL, None),
            MJMAOA: GeckoBoolStructAccessor(self.struct, MJMAOA, BIAMJM, SOOQNR, None),
            JMAOAW: GeckoBoolStructAccessor(self.struct, JMAOAW, MAOAWB, XUJUTY, None),
            AOAWBS: GeckoBoolStructAccessor(self.struct, AOAWBS, MAOAWB, JVHFTH, None),
            OAWBSI: GeckoBoolStructAccessor(self.struct, OAWBSI, MAOAWB, OKPHUO, None),
            AWBSIR: GeckoBoolStructAccessor(self.struct, AWBSIR, MAOAWB, FXQGLR, None),
            WBSIRY: GeckoBoolStructAccessor(self.struct, WBSIRY, MAOAWB, XLSXUJ, None),
            BSIRYX: GeckoBoolStructAccessor(self.struct, BSIRYX, MAOAWB, EFXQGL, None),
            SIRYXB: GeckoBoolStructAccessor(self.struct, SIRYXB, MAOAWB, RJHIUS, None),
            IRYXBQ: GeckoBoolStructAccessor(self.struct, IRYXBQ, MAOAWB, SOOQNR, None),
            RYXBQF: GeckoBoolStructAccessor(self.struct, RYXBQF, YXBQFY, XUJUTY, None),
            XBQFYL: GeckoBoolStructAccessor(self.struct, XBQFYL, YXBQFY, JVHFTH, None),
            BQFYLJ: GeckoBoolStructAccessor(self.struct, BQFYLJ, YXBQFY, OKPHUO, None),
            QFYLJU: GeckoBoolStructAccessor(self.struct, QFYLJU, YXBQFY, FXQGLR, None),
            FYLJUI: GeckoBoolStructAccessor(self.struct, FYLJUI, YXBQFY, XLSXUJ, None),
            YLJUIK: GeckoBoolStructAccessor(self.struct, YLJUIK, YXBQFY, EFXQGL, None),
            LJUIKF: GeckoBoolStructAccessor(self.struct, LJUIKF, YXBQFY, RJHIUS, None),
            JUIKFW: GeckoBoolStructAccessor(self.struct, JUIKFW, YXBQFY, SOOQNR, None),
            UIKFWR: GeckoBoolStructAccessor(self.struct, UIKFWR, IKFWRK, FXQGLR, None),
            KFWRKI: GeckoBoolStructAccessor(self.struct, KFWRKI, IKFWRK, XLSXUJ, None),
            FWRKIN: GeckoBoolStructAccessor(self.struct, FWRKIN, IKFWRK, EFXQGL, None),
            WRKINE: GeckoBoolStructAccessor(self.struct, WRKINE, IKFWRK, RJHIUS, None),
            RKINEJ: GeckoBoolStructAccessor(self.struct, RKINEJ, IKFWRK, SOOQNR, None),
        }
